/*  allometry.c
 *   Copyright (C) 2001 Paul Moorcroft, G.C. Hurtt and S.W. Pacala
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include "ed.h"
/* Biomass allometries (KgC)  */
/*****************************************************************************/
/* height(m) diameter(cm) relationships */
/* O'Brien et al  - for 56 pft at BCI                                    */
/*****************************************************************************/
float Dbh(struct cohort **cohortptr, UserData data){
  struct cohort *cc;  
  float dbh;
  float m= 0.64;
  float c= 0.37;

  cc = *cohortptr;
  dbh = pow(10.0,((log10(cc->hite) - c)/m));  
  return(dbh); 
}
/*****************************************************************************/
float Hite(struct cohort **cohortptr, UserData data){
  struct cohort *cc;  
  float h;
  float m= 0.64;
  float c= 0.37;
  cc = *cohortptr;    
  if(cc->dbh<= data->max_dbh[cc->pft]) h = pow(10.0,(log10(cc->dbh)*m + c));
  else h = pow(10.0,(log10(data->max_dbh[cc->pft])*m + c));
  return h; 
}
/****************************************************************************/
/* calc leaf biomass from height dbh and wood density using allometry of    */
/* J.G. Saldarriaga et al 1988 - Rio Negro                                  */
/* Journal of Ecology vol 76 p938-958                                       */
/****************************************************************************/
float Bleaf(struct cohort **cohortptr, UserData data){
  struct cohort *cc;  
  float bleaf;  
  int pft;
 
  cc = *cohortptr;
  pft = cc->pft;

  if(cc->dbh <= data->max_dbh[pft])
    bleaf = 0.0419*pow(cc->dbh,1.56)*pow(data->rho[cc->pft],0.55);         
  else  bleaf = 0.0419*pow(data->max_dbh[pft],1.56)*pow(data->rho[cc->pft],0.55);         
  return bleaf;
}/****************************************************************************/
/* calc stem biomass from height(m) dbh(cm) and wood density(g/cm3)         */
/*  using allometry of */
/* J.G. Saldarriaga et al 1988 - Rio Negro                                  */
/* Journal of Ecology vol 76 p938-958                                       */
/****************************************************************************/
float Bdead(struct cohort **cohortptr, UserData data){
  struct cohort *cc;  
  float bdead;
  int pft;

  cc = *cohortptr;
  pft = cc->pft;
  bdead= 0.06896*pow(cc->hite,0.572)*pow(cc->dbh,1.94)*pow(data->rho[pft],0.931);
  return bdead;
}
/*****************************************************************************/
/* convert changes in structural biomass bs to changes in height             */
/* bs is stem biomass plus strucrual root biomass                            */
/* consistent with Bstem and h-dbh allometries                               */
/*****************************************************************************/
float dHdBd(struct cohort **cohortptr, UserData data){
  struct cohort *cc; 
  float dbddh;
  int pft;

  cc = *cohortptr;
  pft = cc->pft;

  dbddh= 0.06896*0.572*pow(cc->hite,-0.428)*pow(cc->dbh,1.94)*pow(data->rho[pft],0.931);

  return 1.0/dbddh;

}
/*****************************************************************************/
float dDbhdBd(struct cohort **cohortptr, UserData data){
  struct cohort *cc;  
  float dBD_dDBH,dH_dDBH;
  int pft;

  cc = *cohortptr;
  pft = cc->pft;

   dBD_dDBH= 1.94*0.06896*pow(cc->hite,0.572)*pow(cc->dbh,0.94)*pow(data->rho[pft],0.931);
  if(cc->dbh < data->max_dbh[pft]){  
    dH_dDBH = 1.4976*pow(cc->dbh,-0.36);
    dBD_dDBH += 0.572*0.06896*pow(cc->hite,0.572-1.0)*pow(cc->dbh,1.94)*pow(data->rho[pft],0.931)*dH_dDBH;    
  } 
  return 1.0/dBD_dDBH;


}
/*****************************************************************************/
float dDbhdBl(struct cohort **cohortptr, UserData data){
  struct cohort *cc;  
  float dblddbh;
  cc = *cohortptr;
  dblddbh = 1.56*0.0419*pow(cc->dbh,0.56)*pow(data->rho[cc->pft],0.55);        

  return 1.0/dblddbh;
}
/****************************************************************************/
/*******************  END OF FILE *******************************************/
/****************************************************************************/











































