/*  cohort.c
 *   Copyright (C) 2001 Paul Moorcroft, G.C. Hurtt and S.W. Pacala
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include "ed.h"
/***********************************************************************/
/*****              create new cohort                                ***/
/***********************************************************************/
void create_cohort(unsigned int pft, float n, float hite, float dbh, float balive, float bdead, struct patch **patchptr, UserData data){
   
  /*this function is called in model initialization from init_cohorts and read cohort distribution and during the model run from recruitment */
  struct cohort *nc;
  int i;
  
  /*   printf("In create_cohort: tallest %p shortest %p\n",*ptallest,*pshortest); */
  
  nc = (struct cohort *) malloc(sizeof(struct cohort));
  if (nc == NULL) {
    printf("out of memory\n");
    while( getchar() != '\n');
  }
  /* assign cohort attributes */
  nc->siteptr = (*patchptr)->siteptr;
  nc->patchptr = *patchptr;
  nc->pft = pft;
  nc->pt = data->pt[pft];
  nc->gpp=0.0000;
  nc->npp=0.0000;
  nc->npp2=0.0000;
  nc->resp=0.0000;
  nc->gr_resp=0.0000;
  nc->md=0.0000;
  nc->fs_open=1.0000;
  nc->fsn=1.0000;
  nc->fsw=1.0000;
  for(i=0;i<N_CLIMATE;i++) nc->cbr[i]= 99.0;
  for(i=0;i<N_CLIMATE;i++) nc->cbr_bar[i]= 99.0;
  for(i=0;i<=N_OUT;i++){ 
    nc->fsn_a[i]=1.0; 
    nc->fsw_a[i]=1.0; 
    nc->fso_a[i]=1.0; 
  }
  nc->hite = hite;
  nc->dbh = dbh;
  nc->p[0] =0.0;
  nc->p[1] =0.0;
  nc->status =0;

  /* initialize derivatives to zero */
  nc->dndt = 0.0;
  nc->ddbhdt = 0.0;
  nc->dhdt = 0.0;
  nc->dbalivedt = 0.0;
  nc->dbdeaddt = 0.0;
  nc->payment_to_Nfixers = 0.0000;
  
  /* calculate size of biomass compartments */
  nc->bdead = bdead;
  nc->balive= balive;
  nc->b = nc->balive + nc->bdead;
  
 for(i=0;i<12;i++) nc->previous_size[i]= 0.000;/*zero size */

  nc->bl = (1.0/(1.0 + data->q[pft] + data->qsw[nc->pft]*nc->hite))*nc->balive;
  nc->bls=0.0;
  nc->br = (data->q[nc->pft]/(1.0 + data->q[pft] + data->qsw[nc->pft]*nc->hite))*nc->balive;
  nc->bsw = (data->qsw[nc->pft]*nc->hite/(1.0 + data->q[pft] + data->qsw[nc->pft]*nc->hite))*nc->balive;

  nc->b_old = 0.0;
  nc->bincr = 0.0;
  nc->bs  = nc->bsw + nc->bdead;   
  nc->bstem = data->agf_bs*nc->bs;

  nc->babove = nc->bl + data->agf_bs*nc->bs; 
  nc->bbelow = nc->br + (1.0 - data->agf_bs)*nc->bs; 

  /***********************/
  /* Nitrogen in recruit */
  /***********************/ 

  data->c2n_recruit[pft] = nc->b/(nc->balive/data->c2n_leaf[pft] + nc->bdead/data->c2n_stem);
  
  /*printf("pft %d c2n_recruit %f\n",pft,data->c2n_recruit[pft]);*/
  /***********************/
  
  nc->An_o = 0.0;
  nc->An_c = 0.0;
  nc->E_o = 0.0;
  nc->E_c = 0.0;
  nc->water_uptake=0.0;
  nc->n = n; 

#if DEBUG_CREATE_COHORT
  printf("cc: bs= %f bl= %f br= %f bsw %f \n",nc->bs,nc->bl,nc->br,nc->bsw);
  printf("cc: balive= %f bdead = %f \n",nc->balive,nc->bdead); 
  printf("cc: hite = %f dbh = %f \n",nc->hite,nc->dbh); 
  printf("cc: density of pft %d seedlings = %f \n",nc->pft,nc->n);            
#endif

  nc->lai =  (nc->n)*(nc->bl*data->specific_leaf_area[nc->pft])*(1.0/((*patchptr)->area));     
  /*insert cohort*/
  insert_cohort(&nc,&(*patchptr)->tallest,&(*patchptr)->shortest,data);


  return;
}
/**********************************************************************/
/***             terminates cohorts                                ****/
/**********************************************************************/
void terminate_cohorts(struct patch **patchptr, UserData data){
 
  struct patch *cp;      
  struct cohort *cc;      
  struct cohort *nextc;      

  /* printf("ct: cohort termination: \n"); */
  cp = *patchptr;
  cc = cp->tallest;

  while (cc != NULL){       
    nextc = cc->shorter;  
    /***  cohort size is below threshold   ***/
    /***  remove & adjust size relations among remaining cohorts  ***/
    if((cc->n*cc->b) < BTOL){
      /* printf("ct: termination of cohort %p in patch %p\n",cc,cc->patchptr); */
	if (cc->taller == NULL) cp->tallest = cc->shorter;
	else (cc->taller)->shorter = cc->shorter;
	if (cc->shorter == NULL) cp->shortest = cc->taller;
	else (cc->shorter)->taller = cc->taller;
	free (cc);        
    }
    cc = nextc;
  } 
  /* printf("*** end cohort termination: \n"); */ 

  return;
}
/**********************************************************************/
/**********************************************************************/
/***            finds next taller cohort with patch                * **/
/**********************************************************************/
struct cohort *next_taller(struct cohort *current, float *stp){

   /***  starting with shortest tree on the grid, find tree just  ***/
   /***  taller than tree being considered and return its pointer ***/
   while (current != NULL && current->dbh < *stp)
      current = current->taller;
   
   return (current);
}
/**********************************************************************/
/***         spawn new cohorts of jueveniles                        ***/
/**********************************************************************/
void recruitment(unsigned int t, struct patch **patchptr, UserData data){  
  int pft;
  struct cohort *dc; /* dummy cohort used to calc biomass per plant */
  struct patch *cp;
  struct site *cs;
  
  cp = *patchptr;

  
  /* add new cohorts to linked list of cohorts */
  /* dummy cohort is used to figure out biomass */
  for(pft=0;pft<N_PFT;pft++){ 
        dc = (struct cohort *) malloc(sizeof(struct cohort));
	if (dc == NULL) {
	  printf("out of memory\n");
	  while(getchar() != '\n');
	}
        dc->pft = pft; 
        dc->hite = (data->hgt_min[pft]); 
        dc->dbh = Dbh(&dc,data);     
        /* calculate biomass of new individual */
        dc->bdead = Bdead(&dc,data);
        dc->balive= Bleaf(&dc,data)*(1.0 + data->q[pft] + data->qsw[dc->pft]*dc->hite);
        dc->b = dc->balive + dc->bdead;
        dc->n = 0.0;       

#if INTERNAL_RECRUITMENT
        dc->n = cp->area*cp->repro[pft]/dc->b;        
#endif 
#if EXTERNAL_RECRUITMENT
        /* add in external recruitment */
        dc->n += data->seed_rain[dc->pft];
#endif       
#if DEBUG_SPAWN_COHORTS
  printf("sc: tstep %d pft %d repro %f \n",t,pft,cp->repro[pft]); 
  printf("sc: timestep %d .. spawning cohort ... \n",t);
  printf("sc: hite= %f dbh= %f \n",dc->hite,dc->dbh);
  printf("sc: balive= %f bdead = %f \n",dc->balive,dc->bdead);
  printf("sc: n= %f b=%f \n",dc->n,dc->b);
#endif
        if(dc->n*dc->b> BTOL){ create_cohort(dc->pft,dc->n,dc->hite,dc->dbh,dc->balive,dc->bdead,&cp,data);	
          cp->repro[pft] = 0.0;   /* reset reproductive array */        
        }  
        free(dc);        
  }  /* end loop over pfts */
  return;
}
/****************************************************************************/
/***               join similar cohorts                                   ***/
/****************************************************************************/
void fuse_cohorts(struct patch **patchptr, UserData data)
{
  /* fuse cohorts of same pft and similar heights*/
  /* then resort lists by height if a fusion occurred*/
  int i;

  struct patch *cp;
  struct cohort *cc, *nextc, *nextnextc;      
  float newn, newbalive, newbdead, newb, newbl, newbr, newh, newbstem,total_lai,newdbh,newbs,newbls;
  int fusion_took_place =0;
  
  /* printf("*** cohort fusion: \n"); */

  cp=*patchptr;
    cc = cp->tallest;
    while (cc != cp->shortest){       		
      nextc = cc->shorter;  
      nextnextc = nextc->shorter;	  
      /* calculate total lai within adjacent cohorts */ 
      total_lai= cc->lai + nextc->lai;
       
      while((fabs(cc->dbh - nextc->dbh)/(0.5*(cc->dbh + nextc->dbh)) < FUSETOL) && (nextc != cc)){
	newn = cc->n + nextc->n;    /* sum individuals */      
	if((cc->pft == nextc->pft)&&(total_lai<LAI_TOL)&&(newn>0.0)){
#if DEBUG_FUSE_COHORTS
	  printf("fusion: COHORT FUSION TAKING PLACE\n");
	  printf("fusion: fusing c %p with nextc %p\n",cc,nextc);
	  printf("fusion: c %p: pft= %d h= %f n=%f\n",cc,cc->pft,cc->hite,cc->n);
	  printf("fusion: nextc %p: pft= %d h= %f n=%f\n",nextc,nextc->pft,nextc->hite,nextc->n);
#endif
	  fusion_took_place = 1;
		   	  
	  /* update current: weighted average of biomasses*/
          newbalive= (cc->n*cc->balive + nextc->n*nextc->balive)/newn;
          newbdead= (cc->n*cc->bdead + nextc->n*nextc->bdead)/newn;

          newb= (cc->n*cc->b + nextc->n*nextc->b)/newn;
          newbl= (cc->n*cc->bl + nextc->n*nextc->bl)/newn;
          newbls= (cc->n*cc->bls + nextc->n*nextc->bls)/newn;
          newbr= (cc->n*cc->br + nextc->n*nextc->br)/newn;
          newbstem= (cc->n*cc->bstem + nextc->n*nextc->bstem)/newn;
          newbs= (cc->n*cc->bs + nextc->n*nextc->bs)/newn;

          newh= (cc->n*cc->hite + nextc->n*nextc->hite)/newn;
         
	  newdbh =  (cc->n*cc->dbh + nextc->n*nextc->dbh)/newn;

          /*update current: implied characteristics*/			  
	  cc->balive = newbalive;
	  cc->bdead = newbdead;
          cc->bstem = newbstem;
          cc->bs = newbs;
          cc->bl = newbl;
          cc->bls = newbls;
          cc->br = newbr;
          cc->b = newb;
          cc->babove = cc->bl + data->agf_bs*cc->bs;
  	  cc->n = newn;	  
	  cc->hite = newh;	  
	  cc->dbh = newdbh;

            /* set cbr to taller of two cohorts */
          for(i=0;i<N_CLIMATE;i++) cc->cbr[i] = nextc->cbr[i];
	  for(i=0;i<N_CLIMATE;i++) cc->cbr_bar[i] = nextc->cbr_bar[i];
	  /*printf("cf: %p h= %f n= %f bs=%f bl=%f br= %f \n",cc,cc->hite,cc->n,cc->bs,cc->bl,cc->br); */
  
	  /*delete nextc*/  
	  (nextc->taller)->shorter = nextnextc;  
	  if (nextc->shorter == NULL){
	     cp->shortest = nextc->taller;
	  }
	  else{
	    nextnextc->taller = nextc->taller;
	  }
		  
	  /*free memory of nextc*/
	  free (nextc);
#if DEBUG_FUSE_COHORTS
          printf("fusion: RESULT: \n");
	  printf("fusion: cc %p nextc %p\n",cc,nextc);
	  printf("fusion: c %p: pft= %d h= %f n=%f\n",cc,cc->pft,cc->hite,cc->n);
#endif
	} /* end if */

        if(nextnextc!=NULL){
	  nextc=nextnextc;
	  nextnextc = nextc->shorter;
	}
        else 
	  nextc=cc;
	} /*end checking other cohorts loop*/
	  
       if(cc != cp->shortest) cc = cc->shorter;

    } /*end cohort loop*/

  if(fusion_took_place == 1){  /* if fusion(s) occured sort cohorts */
      sort_cohorts(&cp, data);
  } /* end if */
  /*printf("cf: exiting .. \n");*/ 
  return;
}
/**********************************************************************/
/***                 sort cohorts                                   ***/
/**********************************************************************/
void sort_cohorts(struct patch **patchptr, UserData data){  
       
  /* resort cohort lists within each patch by straight insertion */

  struct patch *current_patch;
 
  struct cohort *shortestc; 
  struct cohort *tallestc;
  struct cohort *current_c;
  struct cohort *next_c;

  /* printf("*** resorting following cohort growth/fusion/deletion \n"); */
  
  current_patch= *patchptr;
  /* printf("PATCH address %p\n",current_patch); */
  /*     loop over cohorts */
  tallestc  = NULL;
  shortestc = NULL;
  
  current_c = current_patch->tallest; 
  
  while (current_c != NULL){  /* loop over current list */     
    next_c = current_c->shorter;
    /*  printf("CURRENT COHORT address %p \t %f \n",current_c,current_c->hite); */
    
    insert_cohort(&current_c,&tallestc,&shortestc,data);
    current_c=next_c;
  }
  current_patch->tallest = tallestc;
  current_patch->shortest = shortestc;
  
  /* printf("*** exit resort \n"); */
  return;
}
/***********************************************************************/
/***               insert cohort into linked list                    ***/
/***********************************************************************/
void insert_cohort(struct cohort **pcc, struct cohort **ptallest, struct cohort **pshortest,UserData data){

  struct cohort *tallptr, *shortptr, *icohort;
  float tsp;

  icohort = *pcc; /* assign address to icohort local name */

  /* place in the correct place in the linked list of heights */
  /* begin by finding cohort that is just taller than the new cohort */
  tsp = icohort->dbh;
  tallptr = next_taller(*pshortest, &tsp);

  /* new cohort is tallest */
  if(tallptr==NULL){
    /* new shorter cohort to the new cohort is the old tallest cohort */ 
    shortptr = *ptallest;
    /* new cohort is tallest cohort and next taller remains null */
    *ptallest = icohort;
  }
  /* new cohort is not tallest */
  else{
    /* next shorter cohort to new cohort is the next shorter cohort */
    /* to the cohort just taller than the new cohort */
    shortptr = tallptr->shorter;
    
    /* new cohort becomes the next shorter cohort to the cohort */
    /*  just taller than the new cohort */
    tallptr->shorter = icohort;
  }

  /* new cohort is shortest */ 
  if(shortptr==NULL){
    /*  next shorter reamins null */
    /* cohort is placed at the bottom of the list */
    *pshortest = icohort; 
  } 
  else{
  /* new cohort is not shortest and becomes next taller cohort */
  /* to the cohort just below it as defined in the previous block */
    shortptr->taller = icohort;
  }

  /* assign taller and shorter links for the new cohort */ 
  icohort->taller = tallptr;
  icohort->shorter = shortptr;

#if DEBUG_INSERT_COHORT
   printf("** ic: address of icohort taller = %p \n",icohort->taller);
   printf("** ic: address of icohort shorter = %p \n",icohort->shorter);
   printf("** ic: address of shorter = %p \n",shortptr);
   printf("** ic: address of taller = %p \n",tallptr);
   printf("** ic: address shortest cohort  = %p \n",*pshortest);
   printf("** ic: address tallest cohort = %p \n",*ptallest);
#endif 

  return;
}
/***************************************************************************/
/***************************************************************************/
/***               split cohorts that get too big                        ***/
/***************************************************************************/
void split_cohorts(struct patch **patchptr, UserData data){
  /*check to see if any cohorts are too big, then split them in two
    if they are and assign new cohort a height = height + epsilon*/

  struct patch *cp;
  struct cohort *cc,*copyc;
  float epsilon = 0.0001;    

  cp = *patchptr;
      cc=cp->tallest;
    while(cc != NULL){
      if((cc->lai > LAI_TOL)){
#if DEBUG_SPLIT_COHORTS
	printf("spc: Spliting cohort %p hite %f n %f lai %f in patch %p age %f \n",cc,cc->hite,cc->n,cc->lai,cp,cp->age);
#endif	
        /* halve the density and lai of cohort and then duplicate */
	cc->n *= 0.5;
	cc->lai *= 0.5;	
        copyc = (struct cohort *) malloc(sizeof(struct cohort));
	if (copyc == NULL) {
          printf("sc: out of memory\n");
	  while(getchar() != '\n');
	}
        copy_cohort(&cc,&copyc);
        /* separate slightly */        
	cc->dbh    -= epsilon;
        copyc->dbh += epsilon;
	/*insert copy into linked list */
	insert_cohort(&copyc,&(cp->tallest),&(cp->shortest),data);	
      } /* end if on lai */
      cc=cc->shorter;
    }  /*  end loop over cohorts */
  return;

}
/***************************************************************************/
/***               copy cohorts                                           **/
/***************************************************************************/
void copy_cohort(struct cohort **cc, struct cohort **copyc){

  /*The easier way to do this would be to bitmap it*/
  int i;
  struct cohort *o,*n;

  o = *cc;
  n = *copyc;

  /* printf("copy cohorts\n"); */
   for(i=0;i<N_CLIMATE;i++) n->cbr[i] = o->cbr[i];
   for(i=0;i<N_CLIMATE;i++) n->cbr_bar[i] = o->cbr_bar[i];

   n->pft = o->pft;
   n->pt = o->pt;
   n->n = o->n;                      /* number of individuals in cohort    */
   n->dndt = o->dndt;                /* rate of change of cohort size   */
   n->dbh= o->dbh;                   /* dbh in cm                            */
   n->hite= o->hite;                 /* height in meters                     */
   n->b= o->b;	     	             /* total biomass per indiv              */
   n->babove= o->babove;	     /* total above ground biomass           */
   n->bbelow= o->bbelow;	     /* total below ground biomass           */
   n->balive= o->balive;	     /* total living biomass per indiv       */
   n->bdead= o->bdead;	             /* dead biomass per indiv               */
   n->bsw= o->bsw;                   /* sapwood in stem and roots */
   n->bl= o->bl; 		     /* leaf biomass per indiv               */
   n->bls= o->bls; 		     /* stored foliar biomass per indiv      */
   n->bstem= o->bstem;	             /* stem biomass per indiv               */
   n->bs= o->bs;	             /* stem biomass per indiv               */
   n->br= o->br;	             /* fine root biomass per indiv    */
   n->leaf_area= o->leaf_area;          /* leaf area of plant                   */
   n->lai= o->lai;                /* leaf area index of plant             */
   n->lite= o->lite;               /* light level for cohort               */
   n->gpp= o->gpp;		     /* net primary PER PLANT!(kg/plant/yr)   	*/
   n->npp= o->npp;		     /* net primary PER PLANT!(kg/plant/yr)   	*/
   n->npp2= o->npp2;	     /* net primary PER PLANT!(kg/plant/yr)   	*/
   n->resp= o->resp;	             /* plant respiration      */
   n->An_o= o->An_o;
   n->An_c= o->An_c;
   n->fs_open= o->fs_open; /*fraction of month with stomates open (dimensionless)*/
   n->fsn= o->fsn;     /* degree of n limitation (dimensionless)*/
   n->fsw= o->fsw;     /* degree of water limitation (dimensionless)*/

   n->carbon_balance= o->carbon_balance;     /* relative carbon balance: ratio of bl to bl* */
   n->p[0]= o->p[0];	             /* seed reproduction */
   n->p[1]= o->p[1];	             /* seed reproduction */
   n->status= o->status;             /* growth status of plant */

   /*water fields*/
   n->water_uptake= o->water_uptake;
   n->E_o= o->E_o;
   n->E_c= o->E_c;

   /*nitrogen fields*/
   n->nitrogen_uptake= o->nitrogen_uptake;
   n->N_uptake_pot= o->N_uptake_pot;
   n->N_uptake_shut= o->N_uptake_shut;
   n->fsn= o->fsn;     /* degree of n limitation (dimensionless)*/
   /* variables needed for integration */
   n->dndt= o->dndt;            /* time derivative of cohort size */
   n->dhdt= o->dhdt;            /* time derivative of height */
   n->ddbhdt= o->ddbhdt;        /* time derivative of dbh */
   n->dbalivedt= o->dbalivedt;  /* time derivative of total living biomass */
   n->dbdeaddt= o->dbdeaddt;    /* time derivative of dead biomass */
   n->dbswdt= o->dbswdt;        /* time derivative of living stem biomass */
   n->dbsdt= o->dbsdt;          /* time derivative of total stem biomass */
   n->dbldt= o->dbldt;          /* time derivative of leaf biomass */
   n->dbrdt= o->dbrdt;          /* time derivative of root biomass */
  
  
   /*linked list fields*/
   n->taller= NULL;          /* pointer to next tallest cohort     */
   n->shorter= NULL;         /* pointer to next shorter cohort     */
   n->patchptr= o->patchptr; /* pointer to patch that cohort is in */
   n->siteptr= o->siteptr;   /* pointer to site that cohort is in  */

   return;

}








