/*  disturbance.c
 *   Copyright (C) 2001 Paul Moorcroft, G.C. Hurtt and S.W. Pacala
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include "ed.h"
/**************************************************************************/
void calculate_site_disturbance_rates(unsigned int t, struct site **current_site, UserData data){

  /* computes disturbance rate for site */
  struct site *cs; 
  struct patch *cp;
  int i;
  cs = *current_site;

  /* calculate patch level disturbance rates */
  cp=cs->oldest_patch;
  while (cp != NULL) {  
      calculate_patch_disturbance_rates(t,&cp,data);  
      cp = cp->younger;
    } /* end loop over patches */

  cs->disturbance_rates[0] = data->treefall_disturbance_rate;
  cs->disturbance_rates[1]=0.0;
  for(i=0;i<N_SUB;i++) cs->disturbance_rates[1] += cs->lambda1[i]/(1.0*N_SUB);

  /*Use largest disturbance mode*/
  if(cs->disturbance_rates[1] > cs->disturbance_rates[0]){ 
    cs->disturbance_rate = cs->disturbance_rates[1];
    cs->dist_type=1;
  }
  else{
    cs->disturbance_rate = cs->disturbance_rates[0];
    cs->dist_type=0;
  }
  cs->sdata->loss_fraction[0] = 0.0;
  cs->sdata->loss_fraction[1] = 0.0;

  return;
}
/*************************************************************************/
void calculate_patch_disturbance_rates(unsigned int t, struct patch **current_patch, UserData data){
  struct site *cs;
  struct patch *cp;
  int i;  
  cp= *current_patch;
  cs= cp->siteptr;

  /*printf("cpdr: entering calculate_patch_disturbance_rates\n");*/

  /* CALCULATE DISTURBANCE RATES */
  cp->disturbance_rates[0] = data->treefall_disturbance_rate;

  /* if fires occur at site */ 
  cp->disturbance_rates[1]=0.0;  
  for(i=0;i<N_SUB;i++) {
    /*printf("cpdr: patch %p i %d lambda1i %f\n",cp,i,cp->lambda1[i]);*/
    cp->disturbance_rates[1] += cp->lambda1[i]/(1.0*N_SUB);
  }
  /*Only use larger of two natural disturbance modes !!*/
  if(cp->disturbance_rates[1] > cp->disturbance_rates[0]){
    cp->dist_type =1;   
    cp->disturbance_rate = cp->disturbance_rates[1];
  }
  else{ 
    cp->dist_type =0;   
    cp->disturbance_rate = cp->disturbance_rates[0];
  }

#if DEBUG_DISTURBANCE
    fprintf(stdout,"cpdr: time %f site %s patch %p lambda0= %f lambda1= %f dist_rate %f \n",t*data->deltat,cs->name,cp,cp->disturbance_rates[0],cp->disturbance_rates[1],cp->disturbance_rate);
#endif

  return;
}
/**************************************************************************/
float survivorship_from_disturbance(int q, struct cohort **pcurrentc, UserData data){
  float csurv;
  int pft;
  struct cohort *cc;
  struct site *cs;
  cc = *pcurrentc;
  cs = cc->siteptr;
  pft = cc->pft;

  /*treefalls*/    
  if(q==0){     
    if(cc->hite < data->treefall_hite_threshold){
      csurv =  data->treefall_s_ltht[pft]; 
    } 
    else csurv = data->treefall_s_gtht[pft];      
  }  

  /*fires*/    
  else if(q==1){ 
    csurv = 0.0;
  } /* end if on fires */

  return csurv;

}
/***************************************************************************/
void fire_frequency(unsigned int t, struct site **siteptr, UserData data){
  /*for all linked patches*/
  struct site *cs;
  struct patch *cp;
  struct cohort *cc;
  cs=*siteptr;
#if DEBUG_DISTURBANCE  
   printf("entered update_fuel\n");
#endif

  /*init*/
  cs->fuel=0.00;         
  cs->ignition_rate=0.00;         
  cs->fire_flag=0; /* reset fire flag */
  cp = cs->youngest_patch;
  while(cp != NULL){  
    /* init patch fuel and ignition rate */
    cp->fuel=0.00; 
    cp->ignition_rate=0.00;             
    cc = cp->shortest;
    while(cc != NULL){
	cp->fuel += cc->n*cc->babove;
      cc = cc->taller;
    } /* end loop over cohorts */
    
    /* divide by patch area */
    cp->fuel /=  cp->area;
    
    /* add patch total to site total */
    cs->fuel += cp->fuel*cp->area;
    
    /* calc patch level ignition rate */
    if(cp->water < data->k_fw){ 
      cp->ignition_rate = 1.0;
      cs->fire_flag=1; /* set to 1 if any patch dry enough for fires */
    }
    else{
      cp->ignition_rate = 0.0;
    }      

    /* add patch value into site-level integrated-ignition-probability */
    cs->ignition_rate += cp->ignition_rate*cp->fuel*cp->area;
    
    cp=cp->older;     
  }  /* end loop over patches */

  /* divide by site area */
  cs->fuel /= AREA;
  cs->ignition_rate /= AREA;
  
  /* calculate fire dist rate and add into array of within yr values*/
  cs->lambda1[t%N_SUB] = 0.0;
  cp = cs->youngest_patch;  
  while(cp != NULL){
    cp->lambda1[t%N_SUB] = data->fp1*cs->ignition_rate;
    cs->lambda1[t%N_SUB] += cp->lambda1[t%N_SUB]*cp->area;
    cp=cp->older;     
  }  /* end loop over patches */
  cs->lambda1[t%N_SUB] /= AREA; /* divide by total_area; */

#if DEBUG_DISTURBANCE
  cp = cs->youngest_patch;  
  while(cp != NULL){
    printf("\n\n"); 
    printf("ff: site: %s ir %8.5f fuel %8.5f \n",cs->name,cs->ignition_rate,cs->fuel);
    printf("ff: patch: %p age %6.3f area %6.3f ir %8.5f fuel %8.5f \n",cp,cp->age,cp->area,cp->ignition_rate,cp->fuel);
    printf("ff: m %d ",data->time_period);
    for(i=0;i<N_SUB;i++) printf("l%2d= %6.3f ",i,cp->lambda1[i]);
    printf("\n"); 
    cp=cp->older;     
  }  /* end loop over patches */
  printf("\n");
  printf("ff: %s ir %8.5f fuel %8.5f \n",cs->name,cs->ignition_rate,cs->fuel);
  printf("ff: month %2d ",data->time_period);
  for(i=0;i<N_SUB;i++) printf("l%d = %6.3f ",i,cs->lambda1[i]);  
  printf("\n");
#endif 

  return;

}
/****************************************************************************/










