/*  ed.c
 *   Copyright (C) 2001 Paul Moorcroft, G.C. Hurtt and S.W. Pacala
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include "ed.h"
/**************************************************************************/
int main(){

  UserData data;
	
  struct site *first_site;

  FILE *outfile;
  char filename[STR_LEN];

  /* allocate data storage structure */
  data = (UserData) malloc(sizeof *data);  /* Allocate data memory */ 
  strcpy(data->base_filename,PREFIX);

  /*open an ecosystem dynamics file*/
  strcpy(filename,"../output/");
  strcat(filename,data->base_filename);
  strcat(filename,".ed");
  if(!(outfile=fopen(filename,"w"))){ 
    fprintf(stdout,"isar: Can't open file: %s \n",filename);
    while(getchar() != '\n');
  }
  else{
    fclose(outfile);
  }

  /* initialize site structures */
  init_data(data);
  read_input_data_layers(data);

  first_site=NULL;
  init_sites(&first_site,data);

  if(first_site == NULL){ printf("error no valid sites \n"); return 0; }
  print_site_chars(&first_site,data);
  
#if REGION
  print_region_mask(&first_site,data);  
  print_region_grid_cell_area(&first_site,data);
  print_region_precip(&first_site,data);
  print_region_soil_chars(&first_site,data);
  print_region_mech_An(&first_site,data);
  /* print_region_mech_E(&first_site,data); */
  print_region_temp(&first_site,data);
  print_region_soil_temp(&first_site,data);
#endif
  
  model(&first_site,data,0); /* otherwise run once using input values */ 
  
  return 0;
}
/*****************************************************************************/
void ecosystem_dynamics(unsigned int t, float t1, float t2, struct site **siteptr, UserData data){
    /* integrates variables using stiff ode integator */
    /* then perform patch and cohort deletion and fusion algorithms */
  
  struct site *cs;
  struct patch *cp;
  struct cohort *cc;
  
  FILE *outfile;
  char filename[STR_LEN];
  strcpy(filename,"../output/");
  strcat(filename,data->base_filename);
  strcat(filename,".ed");
#if LONG_ED_FILE
    if(!(outfile=fopen(filename,"a"))){
      fprintf(stdout,"ed: Can't open file: %s \n",filename);
      while(getchar() != '\n');
    }
#else
    if(!(outfile=fopen(filename,"w"))){
      fprintf(stdout,"ed: Can't open file: %s \n",filename);
      while(getchar() != '\n');
    }
#endif

  cs= *siteptr;
 
  if((cs->integration_flag == 0) || (RPT_INTEGRATION == 1) ){
    data->current_site= cs; /* assign current site to site strucutre in UserData structure data (address of site containing patch to be integrated) */ 

    fprintf(outfile,"Ecosystem dynamics for %s %p t=%d t1=%f t2=%f \n",cs->name,cs,t,t1,t2); 
    cp=cs->youngest_patch;
    if(t%COHORT_FREQ==0){    
      phenology(t,&cs,data); 
      fire_frequency(t,&cs,data); /* calc fire frequency */
      while (cp != NULL){
        light_levels(&cp,data);
        cp = cp->older;
      } /* end loop over patches */     
    } /*end t%COHORT_FREQ*/

    calculate_site_disturbance_rates(t,&cs,data);  
    integrate_state_variables(&cs,t,t1,t2,data);   

    /**************************************************/
    /* Reproduction Recruitment and Cohort Dynamics   */
    /**************************************************/
    if(t%COHORT_FREQ == 0){
       fprintf(outfile,"repro  \n");
       reproduction(t,&cs,data); 
       fprintf(outfile,"recruitment and cohort dynamics \n");
       cp=cs->oldest_patch;
       while (cp != NULL){
         recruitment(t,&cp,data);
         terminate_cohorts(&cp,data); 
         sort_cohorts(&cp,data);
         fuse_cohorts(&cp,data);
         split_cohorts(&cp,data);
         cp = cp->younger;
       }    
    } /*end t%COHORT_FREQ*/

      if((t%(PATCH_FREQ) == 0)&&(t>0)){
	calculate_site_disturbance_rates(t,&cs,data);
#if PATCH_FUSION
	fuse_patches(t,&cs,data); 	
#endif
	spawn_patches(t,&cs,data);
        terminate_patches(&cs,data);
      }  /* end t%PATCH_FREQ*/
      
  }/*end if on intregration flags */
  fprintf(outfile,"\n end of ecosystem dynamics\n"); 
  
  fclose(outfile);
  return;
}

/**********************************************************************/
/***              initialize patches within grid cell             ***/
/**********************************************************************/
void init_patches(struct site **siteptr, UserData data){
  
  /*function called during model initialization if not a restart*/
  /*This function creates number of patches specified by N_INIT_PATCHES */

   int i; 
   float age,area,water,fsc,stsc,stsl,ssc,psc,msn,fsn;
   int dist_type;
   struct site *cs;
   struct patch *newp;
   struct patch *cp;
   printf("Initializing patches... \n");
   cs= *siteptr;   

   /***  initialize patch based on input parameters  ***/   
   for(i=0;i<N_INIT_PATCHES;i++){
     printf("Initializing patch %d \n",i);
     /*create_patch(&cs,&newp,p_age,cs->sdata->area/(N_INIT_PATCHES*1.0),data);*/
     dist_type = 0;
     age = 0.0;
     area = AREA/(N_INIT_PATCHES*1.0);
     /*initialize patch water to eqm value in abs of vegetation */
     water= (cs->sdata->soil_depth*cs->sdata->theta_max)*pow(cs->sdata->precip_average/cs->sdata->k_sat,1.0/(2.0*cs->sdata->tau+2.0));
     /*initialize Soil Carbon Pools arbitary values */
     fsc=0.01; stsc=0.01; stsl=0.001; ssc=0.0; psc=0.0; msn=1.0; fsn=1.0;

     create_patch(&cs,&newp,dist_type,age,area,water,fsc,stsc,stsl,ssc,psc,msn,fsn,data);
     init_cohorts(&newp,data);
     if(i==0){
       newp->younger=NULL; 
       newp->older=NULL;
       cs->youngest_patch = newp;
       cs->oldest_patch = newp;
       cp = cs->oldest_patch;
     }
     else{
       newp->older=NULL;
       newp->younger =  cp;
       (cp->older) = newp;
       cp = cp->older;
       cs->oldest_patch = newp;
     }
   }
   return;

}
/**********************************************************************/
/***              initialize new cohort                             ***/
/**********************************************************************/
void init_cohorts(struct patch **patchptr, UserData data){
  /*this function is called only during model initialization from init_patches.*/
  int pft;  
  float n;  
  struct cohort *dc;

  /*initial number of seedlings of each pft in patch */
  n = INIT_DENSITY * (*patchptr)->area;

  /* printf("starting ic: address of tallest = %p \n",*ptallest); */
  /* printf("starting ic: address of shortest = %p \n",*pshortest); */

  /* add new cohorts to linked list of cohorts */
  for(pft=0;pft<N_PFT;pft++){

   /* printf("in ic: address of tallest = %p \n",*ptallest); */
   /*	  printf("in ic: address of shortest = %p \n",*pshortest); */
   /* calculate number of individuals */

  /* create a dummy cohort to figure out size of biomass compartments */  
  dc = (struct cohort *) malloc(sizeof(struct cohort));
  if (dc == NULL) {
    printf("out of memory\n");
    while(getchar() != '\n');
  }
  dc->pft = pft; 
  dc->n = n;
  dc->hite = (data->hgt_min[pft]); 
  dc->dbh = Dbh(&dc,data);     
  /* calculate size of biomass compartments */
  dc->bdead = Bdead(&dc,data);
  dc->balive= Bleaf(&dc,data)*(1.0 + data->q[pft] + data->qsw[dc->pft]*dc->hite);
  dc->b = dc->balive + dc->bdead;

  /* create appropritaely sized cohort then free dummy cohort */
  create_cohort(dc->pft,dc->n,dc->hite,dc->dbh,dc->balive,dc->bdead,patchptr,data);	
  free(dc);

  }  /* end loop over pft */
  /*   printf("exiting ic: address of tallest = %p \n",*ptallest); */
  /*   printf("exiting ic: address of shortest = %p \n",*pshortest);*/
  /* printf("exiting ic: address of shortest next = %p \n",(*pshortest)->shorter); */

  /* ptr to next shorter cohort of conspecifics */
  
  return;
}
/*****************************************************************************/
void model(struct site **first_site, UserData data, int sflag){

  unsigned int t;
  unsigned int tsteps;
  double t1,t2; 
  struct site *siteptr;
  char filename[STR_LEN];
  FILE *outfile;
  strcpy(filename,"../output/");
  strcat(filename,data->base_filename);
  strcat(filename,".status");
  outfile=fopen(filename,"w");
  fclose(outfile);

  printf("****** running model \n"); 

  tsteps = ((int)(TMAX*N_SUB))+1;              
  for(t=0;t<tsteps;t++){ 
    t1 = t*TIMESTEP;
    t2 = (t+1)*TIMESTEP;

    data->time_period = ((int)rint(t1*N_CLIMATE))%N_CLIMATE;

    siteptr=*first_site;
    while(siteptr!=NULL){
      /* printf("model: site %s \n",siteptr->name); */
    	ecosystem_dynamics(t,t1,t2,&siteptr,data);  
	siteptr=siteptr->next_site;
    }
    update_sites(t,first_site,data);

    outfile=fopen(filename,"a");
    fprintf(outfile,"time %f\n",t2);
    fclose(outfile);
    
  } 
 
  return;
}
/************************************************************************/
/************************** END OF FILE *********************************/
/************************************************************************/











