/*  growth.c
 *   Copyright (C) 2001 Paul Moorcroft, G.C. Hurtt and S.W. Pacala
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include "ed.h"
/****************************************************************************/
void Growth_Derivatives(struct patch **pcurrentp, struct cohort **pcurrentc, float time, UserData data){

  float ba_star;  /* target living biomass */
  float dbldbd, dbrdbd, dbswdbd, dhdbd;
  float va,vs,u,h;  
  float salloc;
  int i,pft,index;
  float gr_fract;

  /* compute derivatives */
  struct cohort *cc;  
  struct patch *cp;

  cc = *pcurrentc;
  cp = *pcurrentp;
 
  pft = cc->pft;
  /*************/
  /* mortality */
  /*************/
  (cc->dndt) = -1.0*mortality_rates(&cc,data)*(cc->n);

  /***************************/
  /* growth and reproduction */
  /***************************/

  /* update pool sizes */
  cc->hite = Hite(&cc,data);
  
  h= cc->hite;
  /* printf("growth: chrt %p h= %f dbh= %f ba= %f bd = %f \n",cc,cc->hite,cc->dbh,cc->balive,cc->bdead); */
  
  salloc = (1.0 + data->qsw[pft]*h + data->q[pft]);
  if(cc->status >= 5){
   cc->bl  = 0.0;
    cc->bls  = (cc->balive)/salloc;
  }
  else{
    cc->bl  = (cc->balive)/salloc;
    cc->bls  = 0.0;
  }
  cc->b = cc->bdead + cc->balive;
  cc->br  = data->q[pft]*(cc->balive)/salloc;
  cc->bsw = data->qsw[pft]*h*(cc->balive)/salloc;
  cc->bs  = cc->bsw + cc->bdead;   
  cc->bstem = data->agf_bs*(cc->bsw + cc->bdead);   
  cc->babove  = cc->bstem + cc->bl;   
  cc->bbelow  = (1.0-data->agf_bs)*(cc->bsw + cc->bdead)+cc->br;

  /* printf("growth: chrt %p bl= %f bls= %f br= %f bsw= %f bd= %f \n",cc,cc->bl,cc->bls,cc->br,cc->bsw,cc->bdead);  */

  /* calculate gpp and npp and respiration */
  /* plant_water_and_nitrogen_relations(&cc,data); */
    
  npp(&cc,data);
  
  /* calculate maintenance demands = tissue decay */
  cc->md= data->alpha[pft][2]*cc->bl + data->alpha[pft][3]*cc->br + data->alpha[pft][4]*cc->bls ;
  
  /* calculate projected size of living biomass compartment */  
  ba_star = Bleaf(&cc,data)*(1.0 + data->q[pft] + data->qsw[pft]*h);

  /* calculate excess carbon  (npp - md) */
  cc->carbon_balance = cc->npp - cc->md;

  /* calc carbon balance ratio */
  if((cc->npp_max - cc->md) > 0.0)
  cc->cbr[data->time_period] = cc->carbon_balance/(cc->npp_max - cc->md);
  else cc->cbr[data->time_period] = -99.99;

  cc->cbr_bar[data->time_period] = 0.00;
  for(i=0;i<N_CLIMATE;i++){
    index = (data->time_period + i)%N_CLIMATE;
    cc->cbr_bar[data->time_period] += cc->cbr[index]/(N_CLIMATE*1.0);
  }  

  /* on allometry */
  if((cc->balive >= ba_star)&&(cc->carbon_balance > 0.0)){ 
    /* calc carbon balance ratio */
   
    /* calculate va and vs */ 
    /* fraction of carbon going into active vs structural carbon */ 
    if(cc->dbh<=data->max_dbh[cc->pft]){ /* cap on leaf biomass */
      dbldbd = dDbhdBd(&cc,data)/dDbhdBl(&cc,data); 
      dbrdbd = data->q[pft]*dbldbd;
      dhdbd = dHdBd(&cc,data);
      dbswdbd = data->qsw[pft]*(h*dbldbd + cc->bl*dhdbd);
      u  = 1.0/(dbldbd + dbrdbd + dbswdbd);     
      va = 1.0/(1.0 + u);
      vs = u/(1.0 + u);
      gr_fract = 1.0 - data->r_fract[pft];
    }
    else{
      dbldbd = 0.0; dbrdbd = 0.0; dbswdbd = 0.0;      
      va = 0.0;
      vs = 1.0;
      gr_fract = 1.0 - (data->r_fract[pft] + data->c_fract[pft]);
    } 
  }
  /* off allometry */                                  
  else{             
    dbldbd = 0.0; dbrdbd = 0.0; dbswdbd = 0.0;
    va = 1.0; vs = 0.0;                         
    gr_fract = 1.0; 
  }
  
  /* calculate derivatives of living and dead carbon pools */
  
  cc->dbalivedt= gr_fract*va*cc->carbon_balance;
  cc->dbdeaddt = gr_fract*vs*cc->carbon_balance;
  cc->p[0] = (1.0 - gr_fract)*cc->carbon_balance;
  cc->p[1] = 0.0;
  
  /* calculate change in diameter and height */
  cc->ddbhdt = cc->dbdeaddt*dDbhdBd(&cc,data);
  cc->dhdt = cc->dbdeaddt*dHdBd(&cc,data);
  
  /*measured above ground npp = leaf litter + p[0]/stem litter + above ground wood production + leaf production*/
  cc->npp2 = cc->bl*data->alpha[cc->pft][2] + cc->p[0] + data->agf_bs*cc->dbdeaddt + 0.5*cc->dbalivedt;

#if DEBUG_GROWTH_RATES
  printf("t= %f GD: p %p c %p \n",time,cp,cc);
  printf("t= %f GD: gpp= %f npp= %f \n",time,cc->gpp,cc->npp);
  printf("t= %f GD: resp= %f \n",time,cc->resp); 
  printf("t= %f GD: md= %f \n",time,cc->md);   
  printf("t= %f GD: seed p[0]= %f \n",time,cc->p[0]);
  printf("t= %f GD: h = %f dhdt = %f \n",time,cc->hite,cc->dhdt);
  printf("t= %f GD: dbh = %f ddbhdt = %f \n",time,cc->dbh,cc->ddbhdt);
  printf("t= %f GD: balive = %f dbalivedt = %f \n",time,cc->balive,cc->dbalivedt);
  printf("t= %f GD: bdead = %f dbdeaddt = %f \n",time,cc->bdead,cc->dbdeaddt);
  printf("t= %f GD: n = %f dndt = %f \n",time,cc->n,cc->dndt);
  printf("\n");

#endif


  return;
} 
/***************************************************************************/
void Litter(struct patch **pcurrentp, float time, UserData data){
  /* calculate litter input rates from decay and mortality */
  struct patch *cp;
  struct cohort *cc;
  int pft;

  float fast_litter,struct_litter,fast_litter_n, plant_litter, seed_litter, seed_litter_n;
  cp = *pcurrentp;  

  /*****************/
  /* LITTER INPUTS */
  /*****************/
  /*Inputs from litter*/
  cc=cp->shortest;
  fast_litter = 0.0;
  fast_litter_n = 0.0;
  struct_litter = 0.0;
  
  while(cc != NULL){
    pft=   cc->pft;
#if DEBUG_LITTER
    printf("litter chrt %p bl= %f br= %f bdead= %f n= %f dndt= %f \n",cc,cc->bl,cc->br,cc->bdead,cc->n,cc->dndt);
#endif
    plant_litter = ((cc->md)*cc->n + cc->balive*(-1.0*cc->dndt) )/cp->area; 
    fast_litter += plant_litter;
    fast_litter_n += (1.0/data->c2n_leaf[pft])*plant_litter;
    seed_litter = ((cc->p[0]*data->sd_mort)*cc->n)/cp->area;
    seed_litter_n = (1.0/data->c2n_recruit[pft])*seed_litter;
    fast_litter += seed_litter;
    fast_litter_n += seed_litter_n;

    /*payment to N fixers*/
    if(data->Nfixer[cc->pft] == 1)
    fast_litter += cc->n*cc->payment_to_Nfixers/cp->area;

    struct_litter += cc->bdead*(-1.0*cc->dndt)/cp->area;

     cc=cc->taller;
  }  /* end loop over cohorts */

    cp->litter = fast_litter + struct_litter;
    /* Calc Pool Inputs */
    cp->fsc_in = fast_litter;
    cp->ssc_in = struct_litter;     /*from stem*/
    cp->ssl_in = (data->l2n_stem/data->c2n_stem)*struct_litter;
    cp->fsn_in = fast_litter_n;

#if DEBUG_LITTER
   printf("litter: site %p patch %p \n",cp->siteptr,cp);
   printf("fl= %f sl= %f \n",fast_litter,struct_litter);
   printf("fsc_in= %f ssc_in= %f ssl_in %f fsn_in %f \n",cp->fsc_in,cp->ssc_in,cp->ssl_in,cp->fsn_in); 
   printf("\n");
#endif

  return;
}
/****************************************************************************/
/*********************************************************************/
float nitrogen_demand_function(struct cohort **pcurrentc, struct patch **pcurrentp, float time, UserData data){

  struct patch *currentp;
  struct cohort *currentc;

  float n_demand=0.0;

  currentp = *pcurrentp;
  currentc = *pcurrentc;
    
  Growth_Derivatives(&currentp,&currentc,time,data);

  n_demand += currentc->dbalivedt/data->c2n_leaf[currentc->pft];
  n_demand += currentc->dbdeaddt/data->c2n_stem;
  n_demand += currentc->p[0]/data->c2n_recruit[currentc->pft];
  n_demand += currentc->p[1]/data->c2n_recruit[currentc->pft];
  n_demand += currentc->md/data->c2n_leaf[currentc->pft];


  /* printf("h %f l %f n %f An_o %f bl %f npp %f dbalivedt %f dbdeaddt %f N_demand %f\n",currentc->hite,currentc->lite,currentc->n,currentc->An_o,currentc->bl,currentc->npp,currentc->dbalivedt,currentc->dbdeaddt,n_demand);*/


  return(n_demand);

} 
/****************************************************************************/






