/*  init_data.c
 *   Copyright (C) 2001 Paul Moorcroft, G.C. Hurtt and S.W. Pacala
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include "ed.h"
/***************************************************************************/
/***                initialize parameters                                ***/
/***************************************************************************/
void init_data(UserData data){
  
  int i;  
  printf("init_data... \n");	

  data->number_of_sites = 0; /*initialize site counter*/
  data->deltat = TIMESTEP;
  data->max_patch_age = 500.0; /* age at which patch fusions ceases */

  /* treefall disturbance parameters */  
  data->treefall_disturbance_rate = 0.014;  
  data->treefall_hite_threshold = 10.0;
  for(i=0;i<N_PFT;i++) data->treefall_s_gtht[i] = 0.0;
  for(i=0;i<N_PFT;i++) data->treefall_s_ltht[i] = 0.1;

  /* fire disturbance parameters */
  data->fp1 = 10.0;   /* fuel parameter */
  data->k_fw = 200.0; /* threshold drynesss for fires to occur */   

  /* recruitment parameters */
  for(i=0;i<N_PFT;i++) data->seed_rain[i] = 0.0; /* external recruitment */
  data->sd_mort = 0.95; /* seedling mortality */

  /* slope of density-independent mortality rate with wood density*/
  data->m1 = 0.15; 
  data->rho_max = 0.9;

  /* carbon balance mortality parameters */
  data->m2 = 10.0;
  data->m3 = 20.0; 
 
  /* water parameters */
  data->water1 = 80.0;  /* water coefficent */

  /* nitogen model parameters */
  data->nitrogen1 = 0.5;  /* plant nitrogen shutdown coefficient */
  data->nitrogen2 = 40.0;  /* microbial nitrogen shutdown */
    
  /* light competition parameters */
  data->L_extinct = 0.5; /* light extinction  coefficient as func of LAI */
  data->Rn_extinct = 0.5; /* Net Rad. flux extinction coeff as func of LAI */

  /* decomposition parameters */
  data->l2n_stem=150.0;  
  data->l2n_root=10.0;  
  data->l2n_leaf=10.0;  
  data->c2n_structural=150.0; 
  data->c2n_slow=10.0;        
  data->c2n_passive=10.0;       
  data->c2n_root=10.0;
  data->c2n_stem=150.0;
  
  /************************************/
  /*BIODIVERSITY AXIS PARAMETER VALUES*/
  /************************************/
  /* fraction of dispersal that is global scaled to disturbance scale */
  for(i=0;i<N_PFT;i++) data->m[i]=1.0; 
      
  data->max_dbh[0] = 0.17; 
  data->max_dbh[1] = 68.31; 
  data->max_dbh[2] = 68.31;
  data->max_dbh[3] = 68.31;
    
  /* assign physiological type of plant c3=0,c4=1  */
  data->pt[0] = 1;
  data->pt[1] = 0;
  data->pt[2] = 0;
  data->pt[3] = 0;
    
  for(i=0;i<N_PFT;i++) printf("pft %d phys. type = %d \n",i,data->pt[i]);
  
 /*assign phenology, evergreen =0,"drought-deciduous"=1," */ 
  data->phenology[0] = 1;
  data->phenology[1] = 1;
  data->phenology[2] = 1;
  data->phenology[3] = 1;
  
  /* fine root biomass leaf biomass ratio*/ 
  for(i=0;i<N_PFT;i++) data->q[i]=1.0;

  /* ratio of above gnd stem to total stem (stem plus structural roots) */
  data->agf_bs = 0.7;

  /* assign allocation fractions  */ 
  for(i=0;i<N_PFT;i++) data->r_fract[i] = 0.3;    
  for(i=0;i<1;i++) data->c_fract[i] = 0.7;    
  for(i=1;i<N_PFT;i++) data->c_fract[i] = 0.0;    

  for(i=0;i<N_PFT;i++) data->hgt_min[i]=0.5; /* minimum height (m) */
  
  /*assign wood density*/
  data->rho[0] = 0.53;
  data->rho[1] = 0.53;
  data->rho[2] = 0.71;
  data->rho[3] = 0.9;
    
  /* assign decay rates of carbon pools yr^-1 */
  data->alpha[0][0]=0.0;   /* seeds */
  data->alpha[0][1]=0.0;   /* sapwood  */
  data->alpha[0][2]=2.0;   /* leaf */
  data->alpha[0][3]=2.0;   /* root */
  data->alpha[0][4]=0.0;   /* stored leaf carbon*/
  data->alpha[0][5]=0.0;   /* structural */
  
  /*EARLY SUCCESSIONAL*/
  data->alpha[1][0]=0.0;   /* seeds */
  data->alpha[1][1]=0.0;   /* sapwood  */
  data->alpha[1][2]=1.0;   /* leaf */
  data->alpha[1][3]=1.0;   /* root */
  data->alpha[1][4]=0.0;   /* stored leaf carbon*/
  data->alpha[1][5]=0.0;   /* structural */

  /* MID SUCCESSIONAL */
  data->alpha[2][0]=0.0;   /* seeds */
  data->alpha[2][1]=0.0;   /* sapwood  */
  data->alpha[2][2]=0.5;   /* leaf */
  data->alpha[2][3]=0.5;   /* root */
  data->alpha[2][4]=0.0;   /* stored leaf carbon*/
  data->alpha[2][5]=0.0;   /* structural */

  /* LATE SUCCESSIONAL */
  data->alpha[3][0]=0.0;   /* seeds */
  data->alpha[3][1]=0.0;   /* sapwood  */
  data->alpha[3][2]=0.333; /* leaf */
  data->alpha[3][3]=0.333; /* root */
  data->alpha[3][4]=0.0;   /* stored leaf carbon*/
  data->alpha[3][5]=0.0;   /* structural */

  
  /*assign respiraction rates of carbon pools */
  for(i=0;i<N_PFT;i++){  
    /* taken from Ibis model J.Foley 1996 gbc v10 p603-628 */
    data->beta[i][0]=0.0;  /* seeds */
    data->beta[i][1]=0.0;  /* sapwood  */
    data->beta[i][2]=0.0;  /* leaf */
    data->beta[i][3]=0.2;  /* root */
    data->beta[i][4]=0.0;  /* stored leaf carbon */
  }
  
  /* assign leaf life spans in months*/
  /* This is the reference biodiversity axis parameter */
  data->leaf_life_span[0] = 12.0/data->alpha[0][2];    
  data->leaf_life_span[1] = 12.0/data->alpha[1][2];    
  data->leaf_life_span[2] = 12.0/data->alpha[2][2];    
  data->leaf_life_span[3] = 12.0/data->alpha[3][2];    
  
  /* leaf c2n ratio */
  for(i=0;i<N_PFT;i++){
    data->c2n_leaf[i] = pow(10.0,(1.65 - 0.34*log10(data->leaf_life_span[i])));
    /*convert to c2n (gC/gN)*/
    data->c2n_leaf[i] = 1000.0/data->c2n_leaf[i]*(1.0/C2B); 
    
    /* specific leaf area */
    /* calculate specific leaf area (cm2/g(biomass)) */
    data->specific_leaf_area[i] = pow(10.0,(2.4 - 0.46*log10(data->leaf_life_span[i])));   /* Global Raich et al 94 PNAS pp 13730-13734 */
    /* convert to (m2/kg(carbon) */
    data->specific_leaf_area[i] = 0.1*C2B*data->specific_leaf_area[i];
    printf("pft %d  sla = %f \n",i,data->specific_leaf_area[i]);
  } /* end loop over pfts */
  
  for(i=0;i<N_PFT;i++){ 
     /* sapwood area per unit leaf biomass (m2/kg) */
     /* 3900.0 = leaf area/sapwood area ratio */ 
     data->qsw[i]= (1.0/3900.0)*data->specific_leaf_area[i];  
     printf("pft %d  qsw = %f \n",i,data->qsw[i]);
  }

  return;
}
/*************************************************************************/
/******************** END OF FILE    *************************************/
/*************************************************************************/










