/*  init_model.c
 *   Copyright (C) 2001 Paul Moorcroft, G.C. Hurtt and S.W. Pacala
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include "ed.h"
/***************************************************************************/
/***        model initializaton routines                                 ***/
/***************************************************************************/
void init_sites(struct site **firsts, UserData data){

  int counter;
  int lon,lat;
  char filename[STR_LEN];
  
  int first_site_flag = 1; /*flag for first site*/
  int model_site_flag = 0; /*flag to model site*/
  struct site *new_site, *last_site;
  int i,j;
  FILE *outfile;

  strcpy(filename,data->base_filename);
  strcat(filename,".ed");
  if(!(outfile=fopen(filename,"a"))){
    fprintf(stdout,"is: Can't open file: %s \n",filename);
    while(getchar() != '\n');
  }
  
  printf("is: intializing sites \n");
  fprintf(outfile,"is: intializing sites  \n");
  
  /*go over region*/
  counter=0;
  for(lat=LATMAX;lat>LATMIN;lat--){
    for(lon=LONMIN;lon<LONMAX;lon++){
      
      counter ++;

      model_site_flag = model_site(lat,lon,counter,data);

      if(model_site_flag == 1){

        /*increment counter*/
        data->number_of_sites ++;

	/*allocate memory for new site*/
	new_site = (struct site *) malloc (sizeof(struct site));
	
	if (new_site == NULL) {
	  fprintf(outfile,"is: malloc site: out of memory\n");
	  
	  while(getchar() != '\n');
	}
	fprintf(stdout,"** address of new_site = %p \n",new_site);
	/* assign site attributes */
	sprintf(new_site->name,"lat%dlon%d",lat,lon);
	
	fprintf(outfile,"new site name: %s\n",new_site->name);
	
	new_site->lat = (float) lat-0.5;
	new_site->lon = (float) lon+0.5; /*negative values for West longitude*/
	fprintf(stdout,"new site coordinates : lat %f lon %f\n",new_site->lat,new_site->lon);
	
	
	/*initialize successful integration flag*/
	new_site->integration_flag = 0;
	
	new_site->sdata = (SiteData) malloc(sizeof *(new_site->sdata));  /* Allocate data memory */ 
	
	if (new_site->sdata == NULL) {
	  fprintf(outfile,"is: malloc sitedata: out of memory\n");
	  while(getchar()!= '\n');
	}
	
	new_site->sdata->L_top = 1.0;  /* Light at the top of the canopy*/
	new_site->sdata->Rn_top = 1.0; /* Net Radn flx at the top of the canopy*/         

	/*******************************************************/
	/*reads in soil depth and hydrological characteristics */
	/*******************************************************/
	read_soil_chars(&new_site);          
		
	fprintf(outfile,"is: Reading in environmental data\n");
	
	/**************/
	/*   precip   */
	/**************/	
	read_precip(&new_site);
        	
	/****************/
	/* temperature  */
	/****************/	
	read_temp(&new_site);
			
	/*********************/
	/* soil temperature  */
	/*********************/
	read_soil_temp(&new_site);
	
	/***************************/
	/* read in physiology data */
	/***************************/	
	fprintf(outfile,"Reading in physiology lut...\n");
	read_mechanism_lut(&new_site,data);
	
	/*site area*/
	new_site->sdata->area = data->grid_cell_area[90-lat][180+lon];
	
	/************************************************/
	/* Misc. initialization                         */	 
	/************************************************/
       	new_site->oldest_patch=NULL;
	new_site->youngest_patch=NULL;
	new_site->next_site=NULL;  
	new_site->create_patch = 1;
	
	for(i=0;i<N_PFT;i++) {
	  new_site->pft_biomass[i] = 0.0; 
	  new_site->pft_basal_area[i] = 0.0; 
	}
	new_site->basal_area = 0.0;
	new_site->plant_biomass = 0.0; 
	new_site->plant_ag_biomass = 0.0;
	new_site->lai = 0.0;
	new_site->soil_c = 0.0;
        new_site->disturbance_rate=0.0;
        for(j=0;j<N_DIST_TYPES;j++) new_site->disturbance_rates[j]=0.0;
	/*printf (" creating initial patch \n"); */
	/*   printf ("address of oldest patch  %p youngest patch %p \n",new_site->oldest_patch,new_site->youngest_patch); */
	
	/* check to see if site of interest */
	new_site->soi = is_soi(new_site->lat,new_site->lon,data);

	/* read in inital patch distribution for the site */
#if RESTART
	read_patch_distribution(&new_site,data);	 
#else
	init_patches(&new_site,data);
#endif
	calculate_site_disturbance_rates(0,&new_site,data);
	calculate_patch_disturbance_rates(0,&(new_site->oldest_patch),data);	
	/* add site to linked list */
	if(first_site_flag == 1){/*if first site do this, otherwise link it*/
	  *firsts = new_site;
	  new_site->next_site= NULL;
	  last_site= new_site;
	  first_site_flag = 0; /*reset flag*/
	}
	else{
	  new_site->next_site = NULL;
	  last_site->next_site = new_site;
	  last_site = new_site;
	}
	
	printf("is: %s %p \n",last_site->name,last_site);
	  
      } /*end model_site_flag loop*/
    } /*end loops over grid*/
  } /* end loops over grid */
  
  fclose(outfile);
  
  return;
  
}
/***************************************************************************/
int is_soi(float lat, float lon, UserData data){ /* flag sites of interest */

  struct soi *cur_soi;
  float lat_min,lat_max,lon_min,lon_max;
  int flag=0; 

  /* printf("is_soi...\n"); */

  cur_soi = data->first_soi;
  /* printf("lat %f lon %f soilat %f soilon %f \n",lat,lon,cur_soi->lat,cur_soi->lon); */
  
  lat_min=lat - 0.5;
  lat_max=lat + 0.5;
  lon_min=lon - 0.5;
  lon_max=lon + 0.5;
  
  while(cur_soi!=NULL){
    if((cur_soi->lat>=lat_min)&&(cur_soi->lat<=lat_max)&&(cur_soi->lon>=lon_min)&&(cur_soi->lon<=lon_max)) 
      flag= 1;
    cur_soi=cur_soi->next_soi;
  }
  
  return flag;  
}
/***************************************************************************/
int model_site (int lat,int lon, int counter,UserData data){
  /*This function returns 1 if site should be modeled, zero otherwise*/

  int model_site_flag;

  model_site_flag = 0;
  
#if REGION
  if(data->mask[90-lat][180+lon] == 1)
    model_site_flag = 1;  /*land mask*/
  if((lat>60) && (lon>-55))
    model_site_flag= 0; /*exclude greenland*/
  if((counter % RARIFY_FACTOR) != 0)
    model_site_flag = 0; /*rarify*/
  if(is_soi(lat-0.5,lon+0.5,data))
    model_site_flag = 1; /*retain sois*/

#else
  if(data->mask[90-lat][180+lon] == 1)
    if(is_soi(lat-0.5,lon+0.5,data) == 1)
      model_site_flag = 1;
#endif

  return model_site_flag;
  
}
/***************************************************************************/








