/*  patch.c
 *   Copyright (C) 2001 Paul Moorcroft, G.C. Hurtt and S.W. Pacala
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include "ed.h"
/**********************************************************************/
/*****     SPAWN PATCHES                                           ****/
/**********************************************************************/
void spawn_patches(unsigned int t, struct site **siteptr, UserData data){
  struct site *cs;
  struct patch *cp,*np;
  struct cohort *cc;
  struct cohort *nc;
  float area =0.0;
  float dA;
  int i,q,pft;
  float fast_litter= 0.0, fast_litter_n=0.0, struct_litter= 0.0;

  cs=*siteptr;
  q = cs->dist_type;  
   
#if DEBUG_SPAWN_PATCHES 
  printf("entered sp: \n");
  cp=cs->youngest_patch;
  while(cp!=NULL){
    printf("sp: patch %p age= %f area= %f dist_rate= %f \n",cp,cp->age,cp->area,cp->disturbance_rate); 
    cp=cp->older;
  }
#endif
               
    /* calculate area by summing contributions*/
    area = 0.0;
    cp=cs->oldest_patch;
    while(cp != NULL){
      area +=  cp->area * (1.0 - exp(-1.0*(cp->disturbance_rate) * data->deltat*PATCH_FREQ));
      cp = cp->younger;
    } /* end loop over patches */

    /****************************************/
    /* check to see if above threshold size */
    /****************************************/
    if(area > F_AREA*AREA){
      create_patch(&cs,&np,cs->dist_type,0.0,area,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,data);
 
    /*go through patches*/   
    cp=cs->oldest_patch;
    while(cp != NULL){    
      
	/*compute change in area to donor patch*/
        dA = cp->area * (1.0 - exp(-1.0*(cp->disturbance_rate) * data->deltat*PATCH_FREQ));	
 
	/***************************************************/
	/* INSERT SURVIVORS FROM DISTURBANCE INTO NEW PATCH */
	/***************************************************/
	cc=cp->shortest;
	while(cc!=NULL){
	    
	  /*make new cohort*/
	  nc = (struct cohort *) malloc(sizeof(struct cohort));
	  if (nc == NULL) {
	    printf("out of memory\n");      
	    while(getchar() != '\n');
	  }      
	    
	  /*copy cohort*/
	  copy_cohort(&cc,&nc);
	  
	  /*surviving indivs*/
	  nc->n = survivorship_from_disturbance(q,&cc,data)*cc->n*dA/cp->area;
	  
	  /*insert*/
	  insert_cohort(&nc,&(np->tallest),&(np->shortest),data);	
	  nc->patchptr = np;
	  
	  /* loss of individuals, as this is a number not a density */
	  cc->n -= cc->n*dA/cp->area;
	  
	  cc = cc->taller;      
	} /* end loop over cohorts */
    

#if DEBUG_SPAWN_PATCHES 
        printf("created patch of size %f \n",area);
        printf("contribution from cp is %f \n",dA);
        printf("averaging in state vars \n");
#endif
	/***************************************************************/
	/*   AVERAGE IN PATCH SITE VARIABLES (repro Water, N, etc...)  */
	/***************************************************************/    
	/* repro  averaged in -weighted, repro not a density*/ 
	for(i=0;i<N_PFT;i++)
	  np->repro[i] += cp->repro[i]*dA/cp->area;
      
	/*WATER  averaged in -weighted*/      
	np->water += cp->water*dA/(np->area);
	np->theta += cp->theta*dA/(np->area);
	
	/* C and N  averaged in -weighted*/
	np->fast_soil_C +=  cp->fast_soil_C*dA/(np->area);
	np->fast_soil_N +=  cp->fast_soil_N*dA/(np->area);	
	np->structural_soil_C +=  cp->structural_soil_C*dA/(np->area);	
	np->slow_soil_C += cp->slow_soil_C*dA/(np->area);	
	np->passive_soil_C +=  cp->passive_soil_C*dA/(np->area);	
	np->mineralized_soil_N +=  cp->mineralized_soil_N*dA/(np->area);       
	np->structural_soil_L += cp->structural_soil_L*dA/(np->area);
      
	/* calculate initial value of heterotrophic respiration term */ 
	np->rh += cp->rh*dA/(np->area);
	
	/*************************************/
	/*ACCULMULATE LITTER FROM DISTURBANCE*/
	/*************************************/	
	fast_litter=0.0;       
	struct_litter=0.0; 
	fast_litter_n=0.0;
	cc=cp->shortest;
	while(cc!=NULL){
	  pft = cc->pft;
		
	  fast_litter += (1.0 - cs->sdata->loss_fraction[q])*cc->balive*(1.0-survivorship_from_disturbance(q,&cc,data))*cc->n/cp->area;
	  fast_litter_n += (1.0/data->c2n_leaf[pft])*(1.0 - cs->sdata->loss_fraction[q])*cc->balive*(1.0-survivorship_from_disturbance(q,&cc,data))*cc->n/cp->area;
	  struct_litter += (1.0 - cs->sdata->loss_fraction[q])*cc->bdead*(1.0-survivorship_from_disturbance(q,&cc,data))*cc->n/cp->area;  
	
	  cc=cc->taller;
	}  /* end loop over cohorts */

	/* add to litter flux */
	np->litter =  fast_litter + struct_litter;
      
	/****************************************************************/
	/* LOAD DISTURBANCE LITTER DIRECTLY INTO CARBON AND N POOLS     */ 
	/****************************************************************/    
	np->fast_soil_C +=  fast_litter * dA/np->area; /*lvs rts */ 
	np->structural_soil_C += struct_litter * dA/np->area;    /* stem*/
	np->structural_soil_L += (data->l2n_stem/data->c2n_stem)*struct_litter * dA/np->area;	
	np->fast_soil_N += fast_litter_n * dA/np->area;
	
#if DEBUG_SPAWN_PATCHES
	printf("pd: site %p patch %p \n",cs,np); 
	printf("fl= %f sl= %f \n",fast_litter,struct_litter);
	printf("fsc= %f ssc_in= %f ssl= %f fsn= %f \n",np->fast_soil_C,np->structural_soil_C,np->structural_soil_L,np->fast_soil_N); 
	printf("\n");
#endif
        
	/* update area of donor patch */ 
	cp->area -= dA;

        cp=cp->younger;
     } /* end loop over patches */

     /**************************/
     /***  INSERT NEW PATCH   **/    
     /**************************/	    
      cp = cs->youngest_patch;
      np -> older = cp;
      np -> younger = NULL;
      cp -> younger = np;
      cs->youngest_patch = np;  
      /*fuse then terminate cohorts*/
      fuse_cohorts(&np,data);
      terminate_cohorts(&np,data);

    } /* end if on new patch area */


#if DEBUG_SPAWN_PATCHES /* check area */
  printf("sp: area check \n");
  cp=cs->youngest_patch;
  while(cp!=NULL){
    printf("sp: site %s patch %p  age %f area %f\n",cs->name,cp,cp->age,cp->area); 
    cp=cp->older;
  }
#endif  
  return;
  
}
/*****************************************************************************/
void create_patch(struct site **siteptr, struct patch **pnewp, int dist_type, float age, float area, float water, float fsc, float stsc, float stsl, float ssc, float psc, float msn, float fsn, UserData data){
  
  /* called by read patch distribition during the model initialization */ 
  struct site *cs; 
  struct patch *np;
  int pft;
  int i;  
  cs=*siteptr; /* assign pointer to site */

  np = (struct patch *) malloc (sizeof(struct patch));
  if (np == NULL) printf("out of memory\n");
  /*printf("** address of np = %p \n",np); */

  /*printf (" assigning attributes \n"); */
  /* assign patch attributes */
  np->dist_type = dist_type;
  np->age = age;   
  np->area = area; 
  np->siteptr= cs; /*pointer to parent site*/
  np->water= water;
  np->fast_soil_C= fsc;
  np->structural_soil_C= stsc;
  np->structural_soil_L= stsl;
  np->slow_soil_C= ssc;
  np->passive_soil_C= psc;
  np->mineralized_soil_N= msn;
  np->fast_soil_N= fsn;
  np->ssc_in=0.0;
  np->ssl_in=0.0;
  np->fsc_in=0.0;
  np->fsn_in=0.0;

  for(i=0;i<N_SUB;i++) np->lambda1[i] = 0.0;
  for(i=0;i<N_DIST_TYPES;i++) np->disturbance_rates[i]=0.0;
  np->disturbance_rate=0.0;
  for(pft=0;pft<N_PFT;pft++) np->repro[pft] =0.0; /* initialize birth array to zero */

  np->tallest=NULL;
  np->shortest=NULL;
  np->older=NULL;
  np->younger=NULL;
  np->plant_ag_biomass = 0.0;
  np->plant_biomass = 0.0;
  np->basal_area = 0.0;
  for(pft=0;pft<N_PFT;pft++){
    np->pft_biomass[pft]=0.0;
    np->pft_ag_biomass[pft]=0.0; 
    np->pft_basal_area[pft]=0.0; 
  }   
    
  np->theta= np->water/(cs->sdata->soil_depth*cs->sdata->theta_max);
  np->plant_water_uptake=0.0;
  np->plant_water_demand=0.0;
  np->perc=0.0;
  np->rh=0.0;
  np->nep=0.0;
  np->npp=0.0;
  np->aa_npp2=0.0; 
  np->aa_npp=0.0; 
  np->aa_nep=0.0; 
  np->aa_rh=0.0;
  np->aa_gpp=0.0;

  /* assign elements of integration array */
  np->wa_e = 0;
  np->fsc_e = 1;
  np->stsc_e = 2;
  np->ssc_e = 3;
  np->msn_e = 4;
  np->fsn_e = 5;
  np->psc_e = 6; 
  np->ssl_e = 7;
  np->fstd=0.0;
  np->A=0.0;
  
  (*pnewp) = np;
  /*  cs->oldest_patch = np; */
  /* cs->youngest_patch = np; */
 
   return;
}
/****************************************************************************/
/****      fuse patches                                                   ***/
/****************************************************************************/
void fuse_patches(unsigned int t, struct site **siteptr, UserData data){

  struct site *cs;
  struct patch *cp, *nextp,*tpp,*tmpptr;
  int i,j,stop;
  float norm;
  FILE *outfile;
  char filename[256];
  
  /*ALGORITHM*/
  /*set all fusion flags to true*/
  /*create size profiles*/
  /*goto every patch*/
  /*find next older patch with same dist_type*/
  /*check fusion criterion*/
  /*if within criterion, fuse, otherwise, skip*/

  strcpy(filename,data->base_filename);
  strcat(filename,".fp");
  if(t==0) outfile=fopen(filename,"w");
#if LONG_FP_FILE
  else outfile=fopen(filename,"a");
#else
  else outfile=fopen(filename,"w");    
#endif
  /* loop over patches and create pft size profiles */
  cs=*siteptr;
  cp=cs->youngest_patch;
  fprintf(outfile,"fp: time %f calculating size profiles \n",t*TIMESTEP);
  while(cp != NULL){
    patch_pft_size_profile(&cp,N_DBH_BINS,data);
    cp->fuse_flag = 1;
    cp=cp->older;
  }

#if DEBUG_FUSE_PATCHES
  printf("fp: Before fp list of patches\n");
  print_patch_linked_list(&cs,data);
#endif

  cp = cs->youngest_patch;

  while((cp != NULL) && (cp->older != NULL)){ 
  
    /*find a given patches' fusion candidate*/  
    nextp = cp->older;
    stop=0;
    tpp = NULL;
    while((nextp != NULL) && (stop == 0)){
      if(cp->dist_type == nextp->dist_type){
	stop = 1;
	tpp = nextp;
      }
      nextp = nextp->older;
    }
        
    if(tpp != NULL){   /*found a fusion candidate*/
      /*fusion criterion*/  
      for(i=0;i<N_PFT;i++){          /* loop over pft */
	for(j=0;j<N_DBH_BINS;j++){      /* loop over hgt bins */
	  if((cp->pft_density_profile[i][j] > NTOL) || (tpp->pft_density_profile[i][j] > NTOL)){
	    norm = fabs(cp->pft_density_profile[i][j]-tpp->pft_density_profile[i][j])/(0.5*(cp->pft_density_profile[i][j]+tpp->pft_density_profile[i][j]));
	    /*fprintf(outfile,"fp: n%d%d %f ",i,j,norm); */
	    if(norm > PROFILE_TOL||tpp->age > data->max_patch_age){
	      cp->fuse_flag=0;  /*reject*/
	    }
	  } /* end if on NTOL */
	}
      } /* end: fusion criterion */

      /*fusion*/
      if(cp->fuse_flag == 1){
	fprintf(outfile,"fp: time %f FUSION donorp %p dt %d age %f area %f tpp %p dt %d age %f area %f\n",t*TIMESTEP,cp,cp->dist_type,cp->age,cp->area,tpp,tpp->dist_type,tpp->age,tpp->area);

	tmpptr = cp->older;  /*tmpptr is needed bc f2p frees cp*/
	fuse_2_patches(&cp,&tpp,data);
	cp = tmpptr;

	fprintf(outfile,"fp: time %f FUSION RESULT p %p dt %d age %f area %f\n",t*TIMESTEP,tpp,tpp->dist_type,tpp->age,tpp->area);

      }
      else{
	cp = cp->older;
      }
    
    }/*end if tp patch != NULL*/
    else{
      cp = cp->older;
    }
  } /* end loop over patches */

#if DEBUG_FUSE_PATCHES
  printf("fp: After fp list of patches\n");
  print_patch_linked_list(&cs,data);
#endif

  fprintf(outfile,"fp: exiting fuse patches \n"); 
  fclose(outfile);
  
  return;
  
}
/****************************************************************************/
/****      fuse_2_patches                                                ***/
/****************************************************************************/
void fuse_2_patches(struct patch **patchptr1, struct patch **patchptr2, UserData data){

  /*This function fuses the two patches specified in the argument.
    It fuses the first patch in the argument (the "donor") into the second
    patch in the argument (the "recipient"), and frees the memory 
    associated with the second patch*/

  struct site *cs;
  struct patch *dp, *rp;
  struct cohort *cc, *nextc;
  int i;

  dp = *patchptr1;
  rp = *patchptr2;
 
  cs = rp->siteptr;

#if DEBUG_FUSE_PATCHES             
  /*DONOR PATCH*/
  /*count the number of cohorts */
  cc = dp->shortest;
  ncohorts=0;
  while(cc != NULL){
    ncohorts ++;
    cc=cc->taller;
  }
  fprintf(stdout,"f2p: DONOR BF: id %p dt %d age %f area %f nc= %d\n",dp,dp->dist_type,dp->age,dp->area,ncohorts);
    
  /*RECEIPIENT PATCH*/
  /*count the number of cohorts */
  cc = rp->shortest;
  ncohorts=0;
  while(cc != NULL){
    ncohorts ++;
    cc=cc->taller;
  }
  fprintf(stdout,"f2p: RECIPIENT BF: id %p dt %d age %f area %f nc= %d\n",rp,rp->dist_type,rp->age,rp->area,ncohorts);
    
#endif

  /*area weighted average of ages */
  rp->age = (dp->age * dp->area + rp->age * rp->area)/(dp->area + rp->area);
  
  /*area weighted average of water*/
  rp->water = (dp->water * dp->area + rp->water * rp->area)/(dp->area + rp->area);
  rp->theta = (dp->theta * dp->area + rp->theta * rp->area)/(dp->area + rp->area);
  
  /*area weighted average of soil Carbon and N*/
  rp->fast_soil_C = (dp->fast_soil_C*dp->area + rp->fast_soil_C*rp->area)/(dp->area+rp->area);
  rp->structural_soil_C = (dp->structural_soil_C*dp->area + rp->structural_soil_C*rp->area)/(dp->area+rp->area);
  rp->slow_soil_C = (dp->slow_soil_C*dp->area + rp->slow_soil_C*rp->area)/(dp->area+rp->area);
  rp->passive_soil_C = (dp->passive_soil_C*dp->area + rp->passive_soil_C*rp->area)/(dp->area+rp->area);
  rp->mineralized_soil_N = (dp->mineralized_soil_N*dp->area + rp->mineralized_soil_N*rp->area)/(dp->area+rp->area);
  rp->fast_soil_N = (dp->fast_soil_N*dp->area + rp->fast_soil_N*rp->area)/(dp->area+rp->area);
  rp->structural_soil_L = (dp->structural_soil_L*dp->area + rp->structural_soil_L*rp->area)/(dp->area+rp->area);

  /* average of repro buffers*/
  for(i=0;i<N_PFT;i++)
    rp->repro[i] = (rp->repro[i]*rp->area + dp->repro[i]*dp->area)/(rp->area + dp->area);
  
#if 0  
  /* adjust densities of cohorts in recipient patch */
  cc = rp->shortest;
  while(cc != NULL){
    cc->n *= rp->area/(rp->area + dp->area);
    cc=cc->taller;
  }
  /* adjust densities of cohorts in donor patch */
  cc = dp->shortest;
  while(cc != NULL){
    cc->n *= dp->area/(rp->area + dp->area);
    cc=cc->taller;
  }
#endif
 
  /*update area*/
  rp->area = rp->area + dp->area;
  
  /*insert donor cohorts into recipient patch*/
  cc = dp->shortest;
  if(cc!=NULL) nextc = cc->taller;
  while(dp->shortest!=NULL){
    insert_cohort(&cc,&rp->tallest,&rp->shortest,data);
    cc->patchptr = rp;
    cc= nextc;
    dp->shortest = cc;
    if(cc != NULL) nextc = cc->taller;
  }
 
  
  /* update size profile within patch */
  patch_pft_size_profile(&rp,N_DBH_BINS,data);
  /* update pointers */
  if(dp != dp->siteptr->youngest_patch) /*donor not youngest*/
      dp->younger->older = dp->older;
    else dp->siteptr->youngest_patch = dp->older;
    if(dp != dp->siteptr->oldest_patch)  /*donor not oldest*/
      dp->older->younger = dp->younger;
    else dp->siteptr->oldest_patch = dp->younger;

  /*free donor patch*/
  free(dp);

#if DEBUG_FUSE_PATCHES
  /*count the number of cohorts */
  cc = rp->shortest;
  ncohorts=0;
  while(cc != NULL){
    ncohorts ++;
    cc=cc->taller;
  }
  fprintf(stdout,"f2p: RECIPIENT AF: age %f area %f nc= %d\n",rp->age,rp->area,ncohorts);
  print_cohort_linked_list(&rp,data); 
#endif
  
  return;
  
}
/***************************************************************************/
/***   Terminate Patches                                                  **/
/***************************************************************************/
void terminate_patches(struct site **siteptr, UserData data){
  struct site *cs;
  struct cohort *cc;
  struct patch *cp,*nextpatch;
  int stop;
  float epsilon=0.0;
  
  /*printf("***In terminate_patches\n");*/
  cs=*siteptr;
  cp= cs->oldest_patch;

  if(cp->area/AREA < F_AREA){ 
    epsilon += cp->area/AREA;        

    nextpatch=cp->younger;
    if(cp->older == NULL) {
      cs->oldest_patch = cp->younger;
      cs->oldest_patch->older = NULL;
    }
    else if(cp->younger == NULL){
      cs->youngest_patch = cp->older;
      cs->youngest_patch->younger = NULL;
    }
    else{
      (cp->older)->younger = cp->younger;
      (cp->younger)->older = cp->older;
    }

    /*free cohorts in patch being terminated*/
    cc=cp->shortest;
    if(cc == NULL) stop = 1;
    else stop = 0;
    while(stop == 0){
      if(cc->taller == NULL){
	free(cc);
	stop = 1;
      }
      else{
	cc=cc->taller;
	free(cc->shorter);
      }
    }
    free(cp);
    /*printf("Freed patch\n");*/
  }


  if(epsilon > 0.0){ /* ADJUST AREAS AND OF OTHER PATCHES TO COMPENSATE */
    cp=cs->oldest_patch;
    while(cp != NULL){
      cp->area *= 1.0/(1.0-epsilon);
      cc=cp->tallest;
      while(cc != NULL){
        cc->n *= 1.0/(1.0-epsilon);
        cc=cc->shorter;
      }        
       cp=cp->younger;
    }
  }

  return;
}
/***************************************************************************/
/*** binned patch size profiles                                         **/
/***************************************************************************/
/* PRM: 5/1/99 modified to compare dbh profiles */ 
void patch_pft_size_profile(struct patch **pcp,int nbins, UserData data){

   struct patch *cp;
   struct cohort *cc;

   float min,max,dh;
   int i,j;
   
   /* printf("psp: \n"); */
   cp=*pcp;

   dh=(DBHMAX/N_DBH_BINS);

   /* initialize bins */
   for(i=0;i<N_PFT;i++){  for(j=0;j<N_DBH_BINS;j++){ 
      cp->pft_density_profile[i][j]=0.0;
      cp->pft_basal_area_profile[i][j]=0.0;
      cp->pft_agb_profile[i][j]=0.0;
   }}

   /* printf("psp: cp = %p \n",cp); */
   /* update bins */ 
   cc=cp->shortest;
   while(cc != NULL){      /* loop over cohorts */
     /*      printf("cc = %p dbh = %f pft= %d n= %f \n",cc,cc->dbh,cc->pft,cc->n); */
     for(j=0;j<N_DBH_BINS-1;j++){     /* loop over bins */
        if (j==0) min=0.0;
        else min= j*dh;
        max= (j+1)*dh;
        if((cc->dbh > min) && (cc->dbh <= max)){

          cp->pft_density_profile[cc->pft][j] += cc->n/cp->area;
          cp->pft_basal_area_profile[cc->pft][j] += (M_PI/4.0)*pow(cc->dbh,2.0)*cc->n/cp->area;
          cp->pft_agb_profile[cc->pft][j] += cc->babove*cc->n/cp->area;

        } /* end if */
     } /* end loop over dbh bins*/

     /* deal with largest dbh bin */
     j=N_DBH_BINS-1;
     min= j*dh;
     if(cc->dbh > min){
          cp->pft_density_profile[cc->pft][j] += cc->n/cp->area;
          cp->pft_basal_area_profile[cc->pft][j] += (M_PI/4.0)*pow(cc->dbh,2.0)*cc->n/cp->area;
          cp->pft_agb_profile[cc->pft][j] += cc->babove*cc->n/cp->area;
     } /* end if */
     

     cc=cc->taller;
   } /* end loop over ochorts */   

   /* printf("exiting psp: \n");   */ 
   return;
}
/*************************************************************************/
/***************************************************************************/
void print_patch_linked_list(struct site **siteptr, UserData data){
  struct site *cs;
  struct patch *cp;

  cs= *siteptr;
  cp= cs->youngest_patch;
  printf("ppll: patches: \n"); 
  while (cp != NULL) { /* loop over patches */
  printf("ppll: cp age %f %p area= %f\n",cp->age,cp,cp->area); 
    cp = cp->older;
  } /* end loop over patches */

  printf("pll: cohorts: \n"); 
  cp= cs->youngest_patch;
  while (cp != NULL) { /* loop over patches */
    print_cohort_linked_list(&cp,data);
    cp = cp->older;
  } /* end loop over patches */
  return;
}
/***************************************************************************/
void print_cohort_linked_list(struct patch **patchptr, UserData data){

  struct patch *cp;
  struct cohort *cc;
  cp= *patchptr;
  
  printf("pcll: cp age %f area %f %p : tallest %p shortest %p\n",cp->age,cp->area,cp,cp->tallest, cp->shortest); 
      cc = cp->tallest;
      while(cc != NULL){
	printf("pcll: cohort %p h= %f dbh= %f pft %d n= %f lai= %f taller %p shorter %p \n",cc,cc->hite,cc->dbh,cc->pft,cc->n,cc->lai,cc->taller,cc->shorter);  
        cc=cc->shorter;
      } /* end cohort loop */  
  
  return;
}
/***************************************************************************/





