/*  phenology.c
 *   Copyright (C) 2001 Paul Moorcroft, G.C. Hurtt and S.W. Pacala
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include "ed.h"
#define THETA_CRIT 0.5  /* soil moisture threshold for leaf drop  */ 
#define L_FRACT 0.5     /* fraction of leaves retained after leaf fall */
/************************************************************************/
void phenology(int t, struct site **siteptr, UserData data){

  struct site *cs;
  struct patch *cp;
  struct cohort *cc;
  float leaf_litter;
  int pft;
  
  cs= *siteptr;
  calculate_site_water(&cs,data);
  
  /* check for fire conditions set flag to 1 if true then calculate fuel */
  /* printf("check for fire w = %f \n",cs->water); */

  cp = cs->oldest_patch;  
  while(cp != NULL){    
      /*go to every cohort*/
      cc = cp->tallest;
      while(cc != NULL){

        pft = cc->pft;
	cc->status = 0; /* set status to indicate no leaf drop */   
	

	if(((data->phenology[pft] == 1) && (cp->theta < THETA_CRIT)) || ((data->phenology[pft] == 2) && (cs->sdata->temp[data->time_period] < 10.0))) {
          /* set status to indicate leaf drop */   
	  cc->status = 5; 
#if DEBUG_PHENOLOGY
	  printf("ph: month= %d  site %p patch %p cohort %p setting bl to zero status= %d \n",data->time_period,cs,cp,cc,cc->status); 
	  printf("ph: month= %d  site_name %s patch %p pft %d setting bl to zero status= %d \n",data->time_period,cs->name,cp,cc->pft,cc->status);
#endif

          leaf_litter = (1.0-L_FRACT)*cc->bl*cc->n/cp->area;
          /* add leaffall to patch litter flux terms */           
          cp->litter += leaf_litter;

	  /*add litter to soil pools*/
	  cp->fast_soil_N += leaf_litter / data->c2n_leaf[cc->pft];
	  cp->fast_soil_C += leaf_litter;
	  
	  /*decrement balive for leaf litterfall*/
	  cc->balive -= (1.0 - L_FRACT)*cc->bl;
	  /*move retained leaf matter to virtual leaf pool*/
	  cc->bls = L_FRACT*cc->bl;
	  /*reset leaf pool to zero*/
	  cc->bl = 0.0;
	} /*end if*/
	cc = cc->shorter;
      } /*end while cohort*/
      cp = cp->younger;
    }/*end while patch*/
  
      return;
}
/*****************************************************************************/
















