/*  photosynthesis.c
 *   Copyright (C) 2001 Paul Moorcroft, G.C. Hurtt and S.W. Pacala
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/*mechanism.c*/
/*photosynthesis and evapotranspiration file*/
#include "ed.h"

void plant_respiration(struct cohort **cohortptr, int light_index,  UserData data);
/**************************************************************************/
void npp(struct cohort **cohortptr, UserData data){
  
  /*This function calculates npp, and gpp*/

  int pft, pt;
  struct cohort *cc;
  struct site *cs;
  int light_index, time_index;

  cc= *cohortptr;
  cs = cc->siteptr;
  pft = cc->pft;
  pt = cc->pt;

  /*SET INDICIES*/
  /*light index*/  
  light_index = 0;
  while(cs->sdata->light_levels[light_index][pt] > cc->lite) light_index++;
  /*month index*/
  time_index = data->time_period;

  /* RESPIRATION */
  plant_respiration(&cc,light_index,data);

  /*PHOTOSYNTHESIS**************************/
  /* calc potential and max photosynthesis (KgC/m2/mon) */
  cc->An_o = 0.001*cs->sdata->Ao[light_index][time_index][pt];
  cc->An_max = 0.001*cs->sdata->Ao[0][time_index][pt];
  /*convert from KgC/m2/mon to kgC/yr*/
  cc->An_o *= data->specific_leaf_area[pft]*cc->bl*N_CLIMATE; 
  cc->An_max *= data->specific_leaf_area[pft]*cc->bl*N_CLIMATE; 
  
  /* photosynthesis with stomates shut*/
  cc->An_c = 0.0;
  /*convert from KgC/m2/mon to kgC/yr*/
  cc->An_c *= data->specific_leaf_area[pft]*cc->bl*N_CLIMATE; 
  
  /* nitrogen fixation */
  if(data->Nfixer[cc->pft] == 1) cc->payment_to_Nfixers = data->fraction_of_GPP_to_Nfixers*cc->fs_open*cc->An_o;
  else cc->payment_to_Nfixers = 0.000;
 
  /*CONSEQUENCES OF WATER AND NITROGEN LIMITATION*/
  /*grown fraction of month at potential rate (Fm), and 1-Fm at shut down rates*/
 
   cc->gpp =  cc->fs_open*cc->An_o + (1.0-cc->fs_open)*cc->An_c;
   cc->npp =  cc->gpp - cc->resp;
   cc->gr_resp= cc->npp*GROWTH_RESP;
   cc->npp *= (1.0-GROWTH_RESP);

   cc->gpp_max =  cc->fs_open*cc->An_max + (1.0-cc->fs_open)*cc->An_c;
   cc->npp_max =  cc->gpp_max - cc->resp;
   cc->npp_max *= (1.0-GROWTH_RESP);

#if DEBUG_NPP
    printf("npp: cc %p pft= %d pt= %d\n",cc,cc->pft,cc->pt);
    printf("npp: lgt = %f lind %d tind= %d \n",cc->lite,light_index,time_index);
    printf("npp: Ao = %f Ac %f Eo= %f Ec %f \n",cs->sdata->Ao[light_index][time_index][pt],cs->sdata->Ac[light_index][time_index][pt]);
    printf("npp: An_o %f \n",cc->An_o);
    printf("npp: An_c %f \n",cc->An_c);
    printf("npp: bl %f sla %f \n",cc->bl,data->specific_leaf_area[pft]);
    printf("npp: E_o %f \n",cc->E_o);
    printf("npp: E_c %f \n",cc->E_c);
    printf("npp: water supply %f \n",water_supply);
    printf("npp: fsw %f \n",cc->fsw);
    printf("npp: fs_open %f \n",cc->fs_open);
    printf("npp: resp %f \n",cc->resp);
    printf("npp: gpp %f \n",cc->gpp);
    printf("npp: npp %f \n",cc->npp);
#endif

  return;
  
}
/**************************************************************************/
void plant_respiration(struct cohort **cohortptr, int light_index, UserData data){
  /* calculates respiration of plant compartments (stem and root) */
  /* taken from Foley (Ibis model) 1996 gbc v10 p603-628 */
  struct cohort *cc;
  struct site *cs;
  float r_leaf, r_vleaf, r_stem, r_root;
  int tindex;
  int pft;
  int pt;
  float tf;

  cc= *cohortptr;
  cs = cc->siteptr;
  pft = cc->pft;
  pt = cc->pt;


  /*light index*/  
  light_index = 0;
  while(cs->sdata->light_levels[light_index][pt] > cc->lite){
    light_index++;
  }

  /* time index*/
  tindex = data->time_period;

  tf = -1.0*cs->sdata->Ac[light_index][tindex][pt]/cs->sdata->tf[tindex][pt];

  /* resp rates read in are  in gC/m2(leaf)/mo */
  /* calc resp rates in kgC per plant per yr */
  /*5/1/00 not not actual leaf resp anymore. Read actual leaf resp used is
    included below, we now read in net which includes leaf resp and do some
  down regulation under limiting conds*/
  r_leaf = (0.001*N_CLIMATE)*(-1.0*cs->sdata->Ac[light_index][tindex][pt])*cc->bl*data->specific_leaf_area[pft];

  r_vleaf = data->beta[pft][4]*cc->bls*tf;
  r_stem = data->beta[pft][1]*cc->bsw*tf;
  r_root = data->beta[pft][3]*cc->br*tf;
  

  /* 4/24/00 take out leaf resp here, include in gpp below by reading in net
     easier to deal with down regulation all at once*/
  cc->resp = r_leaf + r_vleaf + r_stem + r_root;

  r_vleaf = data->beta[pft][4]*cc->bls*tf;
  r_stem = data->beta[pft][1]*cc->bsw*tf;
  r_root = data->beta[pft][3]*cc->br*tf;

  cc->resp = r_leaf + r_vleaf + r_stem + r_root;

#if DEBUG_NPP
printf("pr: patch= %p age %f area %f cc %p pft= %d tf= %f resp= %f Rl %f Rvl %f Rs %f Rr %f\n",cc->patchptr,cc->patchptr->age,cc->patchptr->area,cc,cc->pft,tf,cc->resp,r_leaf,r_vleaf,r_stem,r_root);
printf("pr: patch= %p age %f area %f cc %p bl %f br %f bls %f bsw %f \n",cc->patchptr,cc->patchptr->age,cc->patchptr->area,cc,cc->bl,cc->br,cc->bls,cc->bsw);
  printf("pr: chrt %p pft %d tind %d lind %d pt %d  \n",cc,pft,tindex,light_index,pt);
  printf("pr: chrt %p Ac= %f \n",cc,current_site->sdata->Ac[light_index][tindex][pt]);
  printf("pr: chrt %p bl= %f sla = %f rleaf= %f \n",cc,cc->bl,data->specific_leaf_area[pft],r_leaf);
  printf("pr: chrt %p tf= %f \n",cc,tf);
  printf("pr: chrt %p resp %f \n",cc,cc->resp); 
#endif
  return;

}
/***************************************************************************/













