/*  print_output.c
 *   Copyright (C) 2001 Paul Moorcroft, G.C. Hurtt and S.W. Pacala
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include "ed.h"
/***************************************************************************/
/***********  PRINT SITE OF INTEREST (SOI) FILES  **************************/
/***************************************************************************/
/* SITE CHARACTERISTICS                                                    */
/***************************************************************************/
void print_site_chars(struct site **siteptr, UserData data){

  struct site *cs;
  FILE *outfile;
  char filename[STR_LEN];
  float min_precip,max_precip,min_soil_temp,max_soil_temp;
  int j;
  strcpy(filename,"../output/");
  strcat(filename,data->base_filename);
  strcat(filename,".sites");

     outfile=fopen(filename,"w");
  
  fprintf(outfile,"Number_of_sites = %d\n",data->number_of_sites);
  cs = *siteptr;

  while(cs != NULL){
      fprintf(outfile, "%s %p lat= %6.2f lon= %6.2f soi= %1d ",cs->name,cs,cs->lat,cs->lon,cs->soi);
      fprintf(outfile,"l_top= %8.4f Rn_top= %8.4f ",cs->sdata->L_top,cs->sdata->Rn_top);

      fprintf(outfile,"stype= %2d depth= %8.3f ksat= %8.4f theta_max= %6.3f conduct= %6.3f ",cs->sdata->soil_type,cs->sdata->soil_depth,cs->sdata->k_sat,cs->sdata->theta_max,cs->sdata->soil_evap_conductivity);

      fprintf(outfile,"exogenous dist params lambda0= %4.2f lambda1= %4.2f \n",cs->disturbance_rates[0],cs->disturbance_rates[1]);
    
      min_precip=9999.9;
      max_precip=-9999.9;
      min_soil_temp=9999.9;
      max_soil_temp=-9999.9;
            
      for(j=0;j<N_CLIMATE;j++){
	if(min_precip > cs->sdata->precip[j]) min_precip = cs->sdata->precip[j];
	if(max_precip < cs->sdata->precip[j]) max_precip = cs->sdata->precip[j];
	if(min_soil_temp > cs->sdata->soil_temp[j]) min_soil_temp = cs->sdata->soil_temp[j];
	if(max_soil_temp < cs->sdata->soil_temp[j]) max_soil_temp = cs->sdata->soil_temp[j];
	
      } /* end loop over climate values */

      fprintf(outfile,"avgp= %f maxp= %f minp= %f avgst= %f maxst= %f minst= %f ",cs->sdata->precip_average,max_precip,min_precip,cs->sdata->soil_temp_average,max_soil_temp,min_soil_temp);

      fprintf(outfile,"\n");

      cs=cs->next_site;
  } /* end loop over sites */  
  fclose(outfile);
  
  return;
}
/****************************************************************************/
/* DIAGNOSTICS                                                              */
/****************************************************************************/
void print_diagnostics(unsigned int time, struct site **siteptr, UserData data){
  struct site *cs;
  struct patch *cp;
  struct cohort *cc;
  FILE *outfile;
  char filename[STR_LEN];
  int ncohorts,npatches;

  cs=*siteptr;
  strcpy(filename,"../output/");
  strcat(filename,data->base_filename);
  strcat(filename,".");
  strcat(filename,cs->name);
  strcat(filename,".diagnostics");
  if(time==0) outfile=fopen(filename,"w");
  else outfile = fopen(filename,"a");
  
  npatches = 0;
  ncohorts=0;
  cp = cs->youngest_patch;
  while(cp != NULL){
    npatches++;
    cc = cp->shortest;
    while(cc != NULL){
      ncohorts++;
      cc=cc->taller;
    }
    cp = cp->older;
  }
  
  fprintf(outfile,"site %s time %f npatches %d ncohorts %d lambda0 = %f lambda1 = %f \n",cs->name,time*TIMESTEP,npatches,ncohorts,cs->disturbance_rates[0],cs->disturbance_rates[1]);
  fclose(outfile);

  return;
}
/***************************************************************************/
/*** SITE-LEVEL PLANT BIOMASS, BASAL AREA AND LAI                        ***/
/***************************************************************************/
void print_site_biomass(unsigned int time, struct site **siteptr, UserData data)
{
  /* biomass (above and below) summed over all patches*/
  struct site *cs;
  FILE *outfile;
  int i;
  char filename[STR_LEN];
  
  cs=*siteptr;

  /* open print files */
  strcpy(filename,"../output/");
  strcat(filename,data->base_filename);
  strcat(filename,".");
  strcat(filename,cs->name);
  strcat(filename,".biomass");

  if(time==0) outfile=fopen(filename,"w");
  else outfile=fopen(filename,"a");
  /* print to files */
  fprintf(outfile,"%s t= %5.5f total_c %8.5f tb %8.5f tagb %8.5f ba %8.5f lai %8.5f ",cs->name,time*TIMESTEP,cs->total_c,cs->plant_biomass,cs->plant_ag_biomass,cs->basal_area,cs->lai);
  for(i=0;i<N_PFT;i++){
    fprintf(outfile,"pft %d b %8.6f agb %8.6f ba %8.6f ",i ,cs->pft_biomass[i],cs->pft_ag_biomass[i],cs->pft_basal_area[i]); 
  }  
  fprintf(outfile,"\n");
  fclose(outfile);
    
}
/***************************************************************************/
/*** PATCH-LEVEL PLANT BIOMASS, BASAL AREA AND LAI                       ***/
/***************************************************************************/
void print_patch_biomass(unsigned int time, struct site **siteptr, UserData data){
  /* biomass (above and below) summed over all patches*/
  struct patch *cp;
  struct site *cs;
  FILE *outfile;
  int i;
  char filename[STR_LEN];
  float tallest;

  /* print out biomass in each patch */
  /* open print files */
  strcpy(filename,"../output/");
  strcat(filename,data->base_filename);
  strcat(filename,".");
  strcat(filename,cs->name);
  strcat(filename,".patch.biomass");

  cs=*siteptr;  

#if LONG_PATCH_FILE
  if(time==0) outfile=fopen(filename,"w");
  else outfile=fopen(filename,"a");
#else
  outfile=fopen(filename,"w");
#endif
  
  cp = cs->youngest_patch;
  while(cp != NULL){
    /* set hmax to zero in empty patches */
    if(cp->tallest == NULL) tallest = 0.0;
    else tallest = cp->tallest->hite;    
    fprintf(outfile,"%s t= %5.2f pid= %p ar= %e age= %5.2f trk= %d tagb= %7.4f barea= %7.4f hmax= %7.4f lai= %7.4f\t" ,cs->name,time*TIMESTEP,cp,cp->area,cp->age,cp->dist_type,cp->plant_ag_biomass,cp->basal_area,tallest,cp->lai);
    
    for(i=0;i<N_PFT;i++){
      fprintf(outfile,"s%d tb= %8.5f ba= %8.5f ",i,cp->pft_biomass[i],cp->pft_basal_area[i]);
    }
    fprintf(outfile,"\n");
    
    cp=cp->older;
  }  /* end loop over patches */
  
  fclose(outfile);

  return;

}
/***************************************************************************/
/*** SITE-LEVEL CARBON FLUXES                                            ***/
/***************************************************************************/
void print_site_cfluxes(unsigned int time, struct site **siteptr, UserData data){
  struct site *cs;
  FILE *nppfile,*nppfile2;
  char filename[STR_LEN];
  char filename2[STR_LEN];

  cs=*siteptr;

  /*MONTHLY CFLUXES*/
  /* site file */
  strcpy(filename,"../output/");
  strcat(filename,data->base_filename);
  strcat(filename,".");
  strcat(filename,cs->name);
  strcat(filename,".cfluxes");
  if(time==0){
    nppfile=fopen(filename,"w");
  }
  else{
    nppfile=fopen(filename,"a");
  }
  fprintf(nppfile,"%s time %f npp %8.6f rh %8.6f nep %8.6f npp2 %8.6f gpp %8.6f\n",cs->name,time*TIMESTEP,cs->npp,cs->rh,cs->nep,cs->npp2,cs->gpp);   
  fclose(nppfile);


  /************************/
  /*AVERAGE ANNUAL CFLUXES*/
  /************************/
  /*print the annual average once a year at the end of the year*/
  if(data->time_period == (N_CLIMATE-1)){
    strcpy(filename2,"../output/");
    strcat(filename2,data->base_filename);
    strcat(filename2,".");
    strcat(filename2,cs->name);
    strcat(filename2,".aa.cfluxes");    
    if(time==N_CLIMATE-1){
      nppfile2=fopen(filename2,"w");
    }
    else{
      nppfile2=fopen(filename2,"a");
    }
    /* print site level annual averages to file */
    fprintf(nppfile2,"%s time %6.2f npp %8.6f rh %8.6f nep %8.6f npp2 %8.6f gpp %8.6f \n",cs->name,time*TIMESTEP,cs->aa_npp,cs->aa_rh,cs->aa_nep,cs->aa_npp2,cs->aa_gpp);     
    fclose(nppfile2); 
  } /* end if on time period */

}
/***************************************************************************/
/*** PATCH-LEVEL CARBON FLUXES                                           ***/
/***************************************************************************/
void print_patch_cfluxes(unsigned int time, struct site **siteptr, UserData data){
  struct patch *cp;
  struct site *cs;
  FILE *nppfile;
  char filename[STR_LEN];

  cs=*siteptr;

  /*patch file*/
  strcpy(filename,"../output/");
  strcat(filename,data->base_filename);
  strcat(filename,".");
  strcat(filename,cs->name);
  strcat(filename,".patch.cfluxes");

#if LONG_PATCH_FILE
  if(time==0) nppfile=fopen(filename,"w");
  else nppfile=fopen(filename,"a");
#else
  nppfile=fopen(filename,"w");
#endif  
  
  /* print out npp and nep in each patch */
  cp = cs->youngest_patch;
  while(cp != NULL){
    fprintf(nppfile,"%s time %f pid %p dist_type %d area %f age %5.2f npp %8.6f rh %8.6f  nep %8.6f npp2 %8.6f gpp %8.6f\n" ,cs->name,time*TIMESTEP,cp,cp->dist_type,cp->area,cp->age,cp->npp,cp->rh,cp->nep,cp->npp2,cp->gpp);  
    cp=cp->older;     
  }  /* end loop over patches */
  
  fclose(nppfile);

  /******************************/
  /*AVERAGE ANNUAL PATCH CFLUXES*/
  /******************************/
  if(data->time_period == (N_CLIMATE-1)){

    /*natural*/
  strcpy(filename,"../output/");
  strcat(filename,data->base_filename);
    strcat(filename,".");
    strcat(filename,cs->name);
    strcat(filename,".patch.aa.cfluxes");
        
#if LONG_PATCH_FILE
    if(time==N_CLIMATE-1) nppfile=fopen(filename,"w");
    else nppfile=fopen(filename,"a");
#else
    nppfile=fopen(filename,"w");
#endif
    
    /* print patch level annual averages to file */
    cp = cs->youngest_patch;
    while(cp != NULL){
      fprintf(nppfile,"%s time %6.2f pid %p dist_type %d area %f age %5.2f npp %8.6f rh %8.6f nep %8.6f npp2 %8.6f gpp %8.6f\n" ,cs->name,time*TIMESTEP,cp,cp->dist_type,cp->area,cp->age,cp->aa_npp,cp->aa_rh,cp->aa_nep,cp->aa_npp2,cp->aa_gpp);  
      cp=cp->older;     
    }  /* end loop over patches */

    fclose(nppfile);
    
  } /* end if on time period */
    
  return;
}
/****************************************************************************/
/* SITE-LEVEL SIZE PROFILES                                                 */
/****************************************************************************/
void print_site_size_profile(unsigned int t, struct site **pcs, int nbins, UserData data){
  FILE *pssp,*pssp2,*pssp3;
  char filename[STR_LEN], filename2[STR_LEN], filename3[STR_LEN];
  struct site *cs;
  int j;
  cs=*pcs;
  strcpy(filename,"../output/");
  strcat(filename,data->base_filename);
  strcpy(filename2,"../output/");
  strcat(filename2,data->base_filename);
  strcpy(filename3,"../output/");
  strcat(filename3,data->base_filename);
  strcat(filename,".");
  strcat(filename2,".");
  strcat(filename3,".");
  strcat(filename,cs->name);
  strcat(filename2,cs->name);
  strcat(filename3,cs->name);
  strcat(filename,".sprof.n");
  strcat(filename2,".sprof.ba");
  strcat(filename3,".sprof.agb");

  if(t==0){
    pssp=fopen(filename,"w");
    pssp2=fopen(filename2,"w");
    pssp3=fopen(filename3,"w");
  }
  else{
    pssp = fopen(filename,"a");
    pssp2 = fopen(filename2,"a");
    pssp3 = fopen(filename3,"a");
  } 


   fprintf(pssp,"t= %f ",t*TIMESTEP);
   fprintf(pssp2,"t= %f ",t*TIMESTEP);
   fprintf(pssp3,"t= %f ",t*TIMESTEP);

   for(j=0;j<N_DBH_BINS;j++){
     fprintf(pssp,"%2d %8.5f ",j,cs->density_profile[j]);
     fprintf(pssp2,"%2d %8.5f ",j,cs->basal_area_profile[j]);
     fprintf(pssp3,"%2d %8.5f ",j,cs->agb_profile[j]);
   }
   fprintf(pssp,"\n");
   fprintf(pssp2,"\n");
   fprintf(pssp3,"\n");

   fclose(pssp);   
   fclose(pssp2);
   fclose(pssp3);
   return;
}
/****************************************************************************/
/* PATCH-LEVEL SIZE PROFILES                                                */
/****************************************************************************/
void print_patch_size_profile(unsigned int t, struct patch **pcp, int nbins, UserData data){
  FILE *pprofile,*pprofile2,*pprofile3;
  char filename[STR_LEN], filename2[STR_LEN], filename3[STR_LEN];
  struct patch *cp;
  int i,j;
  cp=*pcp;
  strcpy(filename,"../output/");
  strcat(filename,data->base_filename);
  strcpy(filename2,"../output/");
  strcat(filename2,data->base_filename);
  strcpy(filename3,"../output/");
  strcat(filename3,data->base_filename);
  strcat(filename,".");
  strcat(filename2,".");
  strcat(filename3,".");
  strcat(filename,cp->siteptr->name);
  strcat(filename2,cp->siteptr->name);
  strcat(filename3,cp->siteptr->name);

  strcat(filename,".pprof.n");
  strcat(filename2,".pprof.ba");
  strcat(filename3,".pprof.agb");

  if(t==0){
    pprofile=fopen(filename,"w");
    pprofile2=fopen(filename2,"w");
    pprofile3=fopen(filename3,"w");
  }
  else{
    pprofile = fopen(filename,"a");
    pprofile2 = fopen(filename2,"a");
    pprofile3 = fopen(filename3,"a");
  } 
      
   for(i=0;i<N_PFT;i++){
     fprintf(pprofile,"t= %f p= %p dist_type= %d age= %f  ",t*TIMESTEP,cp,cp->dist_type,cp->age);
     fprintf(pprofile2,"t= %f p= %p dist_type= %d age= %f  ",t*TIMESTEP,cp,cp->dist_type,cp->age);
     fprintf(pprofile3,"t= %f p= %p dist_type= %d age= %f  ",t*TIMESTEP,cp,cp->dist_type,cp->age);

     fprintf(pprofile,"pft%2d  ",i);
     fprintf(pprofile2,"pft%2d  ",i);
     fprintf(pprofile3,"pft%2d  ",i);

     for(j=0;j<N_DBH_BINS;j++){
       fprintf(pprofile,"%2d %8.5f  ",j,cp->pft_density_profile[i][j]);
       fprintf(pprofile2,"%2d %8.5f ",j,cp->pft_basal_area_profile[i][j]);
       fprintf(pprofile3,"%2d %8.5f ",j,cp->pft_agb_profile[i][j]);
     }
     fprintf(pprofile,"\n");
     fprintf(pprofile2,"\n");
     fprintf(pprofile3,"\n");
   }

   fclose(pprofile);   
   fclose(pprofile2);
   fclose(pprofile3);
   return;
}
/***************************************************************************/
/***  LIGHT LEVELS WITHIN PATCHES                                        ***/
/***************************************************************************/
void print_patch_light_levels(unsigned int time, struct site **siteptr, UserData data){ 

  struct site *cs;
  struct patch *cp;
  struct cohort *cc;
  FILE *outfile;
  char filename[STR_LEN];

  cs=*siteptr;  
  strcpy(filename,"../output/");  
  strcat(filename,data->base_filename);
  strcat(filename,".");
  strcat(filename,cs->name);
  strcat(filename,".light");  
  
  outfile = fopen(filename,"w");

  cp = cs->youngest_patch;
  while(cp != NULL){    cc = cp->shortest;
    while(cc != NULL){
      fprintf(outfile,"%s time %f pid %p dist %d age %3.2f area %f pft %d b %f n %f hite %f light %f\n",cs->name,time*TIMESTEP,cp,cp->dist_type,cp->age,cp->area,cc->pt,cc->b,cc->n,cc->hite,cc->lite);
      cc = cc->taller;
    }
    cp=cp->older;
  }
  fclose(outfile);

  return;
}
/***************************************************************************/
/*** SITE-LEVEL WATER                                                    ***/
/***************************************************************************/
void print_site_water(unsigned int time, struct site **siteptr, UserData data){
  struct site *cs;
  FILE *outfile;  
  char filename[STR_LEN];

  cs=*siteptr;
  strcpy(filename,"../output/");
  strcat(filename,data->base_filename);
  strcat(filename,".");
  strcat(filename,cs->name);
  strcat(filename,".water");

  if(time==0)
    outfile=fopen(filename,"w");
  else
    outfile=fopen(filename,"a");
    
    fprintf(outfile,"%s t= %f theta= %f w= %f rain= %f uptake= %f demand= %f perc= %f soil_evap= %f \n",cs->name,time*TIMESTEP,cs->theta,cs->water,cs->sdata->precip[data->time_period],cs->plant_water_uptake/AREA,cs->plant_water_demand/AREA,cs->perc,cs->soil_evap);

  fclose(outfile);    
  return;
}
/***************************************************************************/
/*** PATCH-LEVEL WATER                                                     */
/***************************************************************************/
void print_patch_water(unsigned int time, struct site **siteptr, UserData data){

  struct patch *cp;
  struct site *cs;
  FILE *outfile;  
  char filename[STR_LEN];

  cs=*siteptr;
  
  strcpy(filename,"../output/");
  strcat(filename,data->base_filename);
  strcat(filename,".");
  strcat(filename,cs->name);
  strcat(filename,".patch.water");

#if LONG_PATCH_FILE
  if(time==0) outfile=fopen(filename,"w");
  else outfile=fopen(filename,"a");
#else
  outfile=fopen(filename,"w");
#endif

  cp = cs->youngest_patch;
  while(cp != NULL){

    fprintf(outfile,"%s t= %f pid= %p dist_type= %d age= %f theta= %f w= %f rain= %f uptake= %f demand= %f perc= %f \n",cs->name,time*TIMESTEP,cp,cp->dist_type,cp->age,cp->theta,cp->water,cs->sdata->precip[data->time_period],cp->plant_water_uptake/cp->area,cp->plant_water_demand/cp->area,cp->perc);

    cp=cp->older;
  }
  
  fclose(outfile);

  return;
}
/***************************************************************************/
/* SITE-LEVEL BELOW-GROUND CARBON AND NITROGEN POOLS                       */
/***************************************************************************/
void print_site_soil_pools(unsigned int time, struct site **siteptr, UserData data)
{
  /* soil carbon pools summed over all patches*/
  struct site *cs;
  FILE *outfile;
  char filename[STR_LEN];

  cs=*siteptr;
  
  /************/
  /*SITE TOTAL*/
  /************/
  strcpy(filename,"../output/");
  strcat(filename,data->base_filename);
  strcat(filename,".");
  strcat(filename,cs->name);
  strcat(filename,".sc");
    
  if(time==0) outfile=fopen(filename,"w");
  else outfile=fopen(filename,"a");

/* print to files */
  fprintf(outfile,"%s t= %f soil_c %8.6f fsc %8.6f stsc %8.6f stsl %8.6f ssc %8.6f fsn  %8.6f msn  %8.6f \n",cs->name,time*TIMESTEP,cs->soil_c,cs->fast_soil_C,cs->structural_soil_C,cs->structural_soil_L,cs->slow_soil_C,cs->fast_soil_N,cs->mineralized_soil_N);
  
  fclose(outfile);
  return;
}
/***************************************************************************/
/* PATCH-LEVEL BELOW-GROUND CARBON AND NITROGEN POOLS                      */
/***************************************************************************/
void print_patch_soil_pools(unsigned int time, struct site **siteptr, UserData data){
  struct patch *cp;
  struct site *cs;
  FILE *outfile;
  char filename[STR_LEN];

  cs=*siteptr;
  strcpy(filename,"../output/");
  strcat(filename,data->base_filename);
  strcat(filename,".");
  strcat(filename,cs->name);
  strcat(filename,".patch.sc");

#if LONG_PATCH_FILE
  if(time==0) outfile=fopen(filename,"w");
  else outfile=fopen(filename,"a");
#else
  outfile=fopen(filename,"w");
#endif

  /* print out soil carbon in each patch */
  cp = cs->youngest_patch;
  while(cp != NULL){
    fprintf(outfile,"%s t= %f pid %p area %f age %5.2f soil_c %8.6f A %6.3f fstd %6.3f fsc %8.6f stsc %8.6f stsl %8.6f ssc %8.6f fsn  %8.6f msn  %8.6f \n",cs->name,time*TIMESTEP,cp,cp->area,cp->age,cp->soil_c,cp->A,cp->fstd,cp->fast_soil_C,cp->structural_soil_C,cp->structural_soil_L,cp->slow_soil_C,cp->fast_soil_N,cp->mineralized_soil_N);  
        
    cp=cp->older;     
  }  /* end loop over patches */
  
  fclose(outfile); 
  return;

}
/****************************************************************************/
/***    PATCH AREAS AND DISTURBANCE RATES                                 ***/
/****************************************************************************/
void print_patches(unsigned int time, struct site **siteptr, UserData data)
{

  struct site *cs;
  struct patch *cp;
  struct cohort *cc;
  FILE *outfile;
  char filename[STR_LEN];
  int ncohorts;

  cs=*siteptr;
  strcpy(filename,"../output/");
  strcat(filename,data->base_filename);
  strcat(filename,".");
  strcat(filename,cs->name);
  strcat(filename,".patch");

#if LONG_PATCH_FILE
  if(time==0) outfile=fopen(filename,"w");
  else outfile=fopen(filename,"a");
#else
  outfile=fopen(filename,"w");
#endif
  
  cp = cs->youngest_patch;
  while(cp != NULL){
    ncohorts=0;
    cc = cp->shortest;
    while(cc != NULL){
      ncohorts ++;
      cc=cc->taller;
    }
    fprintf(outfile, "%s t= %f p= %p trk= %d age= %f area= %f l0= %f l1= %f nchrt= %d lai= %f \n",cs->name,((float)(time))*TIMESTEP,cp,cp->dist_type,cp->age,cp->area,cp->disturbance_rates[0],cp->disturbance_rates[1],ncohorts,cp->lai);
    cp = cp->older;
  }
  fclose(outfile);
  
  return;
}
/*****************************************************************************/
/* COHORT ATTRIBUTE FILE                                                     */
/*****************************************************************************/
void print_cohorts(unsigned int time,struct site **siteptr,UserData data)
{

  struct patch *cp;
  struct cohort *cc;
  struct site *cs;
  FILE *cohortfile;
  char filename[STR_LEN];

  cs=*siteptr;
  strcpy(filename,"../output/");  
  strcat(filename,data->base_filename);
  strcat(filename,".");
  strcat(filename,cs->name);
  strcat(filename,".cohorts");

#if LONG_COHORT_FILE
  if(time==0) cohortfile=fopen(filename,"w");
  else cohortfile=fopen(filename,"a");
#else
  cohortfile=fopen(filename,"w");
#endif

  cp = cs->oldest_patch;
  while (cp != NULL){
    cc = (cp->tallest);
    while (cc != NULL){         
      fprintf(cohortfile,"%s t= %f p %p dist_type %d age %f ar %f c %p h %f dbh %f pft %d sta %d n %f ba %f bd %f bl %f bls %f br %f bsw %f gpp %f npp %f resp %f cbr %f Ano %f Anc %f lai %f md %f fso %f r= %f dhdt %f ddbhdt %f dndt %f Eo %f Ec %f wu %f fsw %f gr_resp %f \n",cs->name,time*TIMESTEP,cp,cp->dist_type,cp->age,cp->area,cc,cc->hite,cc->dbh,cc->pft,cc->status,cc->n,cc->balive,cc->bdead,cc->bl,cc->bls,cc->br,cc->bsw,cc->gpp,cc->npp,cc->resp,cc->cbr[data->time_period],cc->An_o,cc->An_c,cc->lai,cc->md,cc->fs_open,cc->p[0],cc->dhdt,cc->ddbhdt,cc->dndt/cc->n,cc->E_o,cc->E_c,cc->water_uptake,cc->fsw,cc->gr_resp);       
      cc = cc->shorter;
    } 
    cp = cp->younger;
  }
  fclose(cohortfile);
  

  return;
}
/*****************************************************************************/
/* SYSTEM STATE                                                              */
/*****************************************************************************/
void print_system_state(unsigned int time,struct site **siteptr,UserData data)
{   /* print state of model to file */ 
  struct cohort *cc;
  struct patch *cp;
  struct site *cs;
  FILE *pfile, *cfile;
  char filename[STR_LEN];
  char filename2[STR_LEN];
  cs=*siteptr;
  
   /* print patch distribution */ 
  strcpy(filename,"../output/");
  strcat(filename,data->base_filename);
  strcat(filename,".");
  strcat(filename,cs->name);
  strcat(filename,".pss");

  pfile=fopen(filename,"w");
  fprintf(pfile,"time patch trk age area water fsc stsc stsl ssc psc msn fsn \n");

  cp = cs->oldest_patch;
  while (cp != NULL){
    fprintf(pfile,"%6.3f %p %d %8.4f %8.4f %8.4f %8.4f %8.4f %8.4f %8.4f %8.4f %8.4f %8.4f \n",time*TIMESTEP,cp,cp->dist_type,cp->age,cp->area,cp->water,cp->fast_soil_C,cp->structural_soil_C,cp->structural_soil_L,cp->slow_soil_C,cp->passive_soil_C,cp->mineralized_soil_N,cp->fast_soil_N);

    cp = cp->younger;
  } /* end loop over patches */
  fclose(pfile);

  /* print cohort distribution */
  strcpy(filename2,"../output/");
  strcat(filename2,data->base_filename);
  strcat(filename2,".");
  strcat(filename2,cs->name);
  strcat(filename2,".css");

  cfile=fopen(filename2,"w");
  fprintf(cfile,"time patch cohort dbh hite pft n bdead balive \n");

  cp = cs->oldest_patch;
  while (cp != NULL){
    cc = (cp->tallest);
    while (cc != NULL){  
      fprintf(cfile,"%6.3f %p %p %8.4f %8.4f %2d %8.4f %8.4f %8.4f \n",time*TIMESTEP,cp,cc,cc->dbh,cc->hite,cc->pft,cc->n,cc->bdead,cc->balive);
      cc = cc->shorter;
    }
    cp = cp->younger;    
  } /* end loop over patches */
  fclose(cfile);
  
  return;
}
/*******************************************************************/
#if REGION
/*******************************************************************/
/*****************  REGIONAL PRINTING ROUTINES *********************/
/*******************************************************************/
/*******************************************************************/
/********  DIAGNOSTICS: NUMBER OF PATCHES AND COHORTS      *********/
/*******************************************************************/
void print_region_diagnostics(unsigned int t,struct site **siteptr, UserData data){
  int i,j;
  struct site *cs;
  struct patch *cp;
  struct cohort *cc;
  FILE *outfile1,*outfile2;
  char filename1[STR_LEN], filename2[STR_LEN];
  int lat_index,lon_index;
  int ncohorts,npatches;
  
  int image_array1[(LATMAX)-(LATMIN)][(LONMAX)-(LONMIN)];
  int image_array2[(LATMAX)-(LATMIN)][(LONMAX)-(LONMIN)];

  if(t%PRINT_OUTPUT_FREQ==0){
    printf("PRINT_REGION_DIAGNOSTICS...");
    strcpy(filename1,"../output/");
    strcat(filename1,data->base_filename);
    strcat(filename1,".region.npatches");    
    strcpy(filename2,"../output/");
    strcat(filename2,data->base_filename);
    strcat(filename2,".region.ncohorts");  
  
    if(t==0){
    outfile1=fopen(filename1,"w");
    outfile2=fopen(filename2,"w");
    }
    else{
      outfile1=fopen(filename1,"a");
      outfile2=fopen(filename2,"a");
    }

  for(i=0;i<((LATMAX)-(LATMIN));i++){
    for(j=0;j<((LONMAX)-(LONMIN));j++){
      image_array1[i][j] = -1;
      image_array2[i][j] = -1;
    }
  }
  
  cs= *siteptr;
  
  while(cs!=NULL){
    
    lat_index = (int) LATMAX-(cs->lat+0.5);
    lon_index = (int) (cs->lon-0.5) - (LONMIN);
    
    npatches = 0;
    ncohorts = 0;
    cp=cs->oldest_patch;
    while(cp != NULL){
      npatches++;
      cc = cp->shortest;
      while(cc != NULL){
        ncohorts++;
	
	cc = cc->taller;
      }    
      cp = cp->younger;
    }
    image_array1[lat_index][lon_index] = npatches;
    image_array2[lat_index][lon_index] = ncohorts;
    cs=cs->next_site;
  }
  
  for(i=0;i<((LATMAX)-(LATMIN));i++){
    for(j=0;j<((LONMAX)-(LONMIN));j++){
     fprintf(outfile1,"%d ",image_array1[i][j]);
     fprintf(outfile2,"%d ",image_array2[i][j]);
    }
    fprintf(outfile1,"\n");
    fprintf(outfile2,"\n");
  }
  fclose(outfile1);
  fclose(outfile2);
  
  printf("DONE\n");
  }
  return;
}
/*****************************************************************************/
/**** REGIONAL SOIL WATER AND MOSTURE                                      ***/
/*****************************************************************************/
void print_region_water(unsigned int t, struct site **siteptr, UserData data){
  int i,j;
  struct site *cs;
  FILE *outfile1,*outfile2;
  char filename1[STR_LEN], filename2[STR_LEN];
  int lat_index,lon_index;
  
  float image_array1[(LATMAX)-(LATMIN)][(LONMAX)-(LONMIN)];
  float image_array2[(LATMAX)-(LATMIN)][(LONMAX)-(LONMIN)];
  
  printf("PRINT_REGION_SOIL_WATER...");
  strcpy(filename1,"../output/");
  strcat(filename1,data->base_filename);
  strcat(filename1,".region.water");    
  strcpy(filename2,"../output/");    
  strcat(filename2,data->base_filename);
  strcat(filename2,".region.theta");    
  
  if(t%PRINT_OUTPUT_FREQ == 0){
    outfile1=fopen(filename1,"w");
    outfile2=fopen(filename2,"w");
  }
  else{
    outfile1=fopen(filename1,"a");
    outfile2=fopen(filename2,"a");
  }
  
  for(i=0;i<((LATMAX)-(LATMIN));i++){
    for(j=0;j<((LONMAX)-(LONMIN));j++){
      image_array1[i][j] = -9.9;
      image_array2[i][j] = -9.9;
    }
  }

  cs= *siteptr;
 
  while(cs!=NULL){

    lat_index = (int) LATMAX-(cs->lat+0.5);
    lon_index = (int) (cs->lon-0.5) - (LONMIN);

    image_array1[lat_index][lon_index] = cs->water;
    image_array2[lat_index][lon_index] = cs->theta;

    cs=cs->next_site;
  }
  
  for(i=0;i<((LATMAX)-(LATMIN));i++){
    for(j=0;j<((LONMAX)-(LONMIN));j++){
     fprintf(outfile1,"%f ",image_array1[i][j]);
     fprintf(outfile2,"%f ",image_array2[i][j]);
    }
    fprintf(outfile1,"\n");
    fprintf(outfile2,"\n");
  }
  fclose(outfile1);
  fclose(outfile2);
  
  printf("DONE\n");

  return;
}
/*****************************************************************************/
/*** REGIONAL WATER FLUXES: PERC, EVAP, TRANSP                             ***/
/*****************************************************************************/
void print_region_water_losses(unsigned int t, struct site **siteptr, UserData data){
  int i,j;
  struct site *cs;
  FILE *outfile1,*outfile2,*outfile3;
  char filename1[STR_LEN],filename2[STR_LEN],filename3[STR_LEN];
  int lat_index,lon_index;
  
  float image_array1[(LATMAX)-(LATMIN)][(LONMAX)-(LONMIN)];
  float image_array2[(LATMAX)-(LATMIN)][(LONMAX)-(LONMIN)];
  float image_array3[(LATMAX)-(LATMIN)][(LONMAX)-(LONMIN)];

  printf("PRINT_REGION_WATER_LOSSES...");
  strcpy(filename1,"../output/");
  strcat(filename1,data->base_filename);
  strcat(filename1,".region.percolation");    
  strcpy(filename2,"../output/");    
  strcat(filename2,data->base_filename);
  strcat(filename2,".region.evaporation");    
  strcpy(filename3,"../output/");  
  strcat(filename3,data->base_filename);
  strcat(filename3,".region.transpiration");    
  
  if(t%PRINT_OUTPUT_FREQ == 0){
    outfile1=fopen(filename1,"w");
    outfile2=fopen(filename2,"w");
    outfile3=fopen(filename3,"w");
  }
  else{
    outfile1=fopen(filename1,"a");
    outfile2=fopen(filename2,"a");
    outfile3=fopen(filename3,"a");
  }
  
  for(i=0;i<((LATMAX)-(LATMIN));i++){
    for(j=0;j<((LONMAX)-(LONMIN));j++){
      image_array1[i][j] = -9.9;
      image_array2[i][j] = -9.9;
      image_array3[i][j] = -9.9;
    }
  }

  cs= *siteptr;
 
  while(cs!=NULL){

    lat_index = (int) LATMAX-(cs->lat+0.5);
    lon_index = (int) (cs->lon-0.5) - (LONMIN);

    image_array1[lat_index][lon_index] = cs->perc;
    image_array2[lat_index][lon_index] = cs->soil_evap;
    image_array3[lat_index][lon_index] = cs->plant_water_uptake;

    cs=cs->next_site;
  }
  
  for(i=0;i<((LATMAX)-(LATMIN));i++){
    for(j=0;j<((LONMAX)-(LONMIN));j++){
     fprintf(outfile1,"%f ",image_array1[i][j]);
     fprintf(outfile2,"%f ",image_array2[i][j]);
     fprintf(outfile3,"%f ",image_array3[i][j]);
    }
    fprintf(outfile1,"\n");
    fprintf(outfile2,"\n");
    fprintf(outfile3,"\n");
  }
  fclose(outfile1);
  fclose(outfile2);
  fclose(outfile3);
  
  printf("DONE\n");

  return;
}
/****************************************************************************/
/**     REGIONAL CARBON FLUXES                                           ****/
/****************************************************************************/
void print_region_carbon_fluxes(unsigned int t, struct site **siteptr, UserData data){
  int i,j;
  struct site *cs;
  struct patch *cp;
  FILE *outfile1,*outfile2,*outfile3,*outfile4,*outfile5;
  char filename1[STR_LEN],filename2[STR_LEN],filename3[STR_LEN],filename4[STR_LEN],filename5[STR_LEN];
  int lat_index,lon_index;
  
  float image_array1[(LATMAX)-(LATMIN)][(LONMAX)-(LONMIN)];
  float image_array2[(LATMAX)-(LATMIN)][(LONMAX)-(LONMIN)];
  float image_array3[(LATMAX)-(LATMIN)][(LONMAX)-(LONMIN)];
  float image_array4[(LATMAX)-(LATMIN)][(LONMAX)-(LONMIN)];

  printf("PRINT_REGION_CARBON_FLUXES...");
  strcpy(filename1,"../output/");
  strcat(filename1,data->base_filename);
  strcat(filename1,".region.npp");    
  strcpy(filename2,"../output/");
  strcat(filename2,data->base_filename);
  strcat(filename2,".region.rh");    
  strcpy(filename3,"../output/"); 
  strcat(filename3,data->base_filename);
  strcat(filename3,".region.nep");    
  strcpy(filename4,"../output/");
  strcat(filename4,data->base_filename);
  strcat(filename4,".region.npp2");    
   
  if(t%PRINT_OUTPUT_FREQ == 0){
    outfile1=fopen(filename1,"w");
    outfile2=fopen(filename2,"w");
    outfile3=fopen(filename3,"w");
    outfile4=fopen(filename4,"w");
  }
  else{
    outfile1=fopen(filename1,"a");
    outfile2=fopen(filename2,"a");
    outfile3=fopen(filename3,"a");
    outfile4=fopen(filename4,"a");
  }

  for(i=0;i<((LATMAX)-(LATMIN));i++)
    for(j=0;j<((LONMAX)-(LONMIN));j++){
      image_array1[i][j] = -99.9;
      image_array2[i][j] = -99.9;
      image_array3[i][j] = -99.9;
      image_array4[i][j] = -99.9;
    }
  
  cs= *siteptr;
  
  while(cs!=NULL){

    lat_index = (int) LATMAX-(cs->lat+0.5);
    lon_index = (int) (cs->lon-0.5) - (LONMIN);
   
    image_array1[lat_index][lon_index] = cs->npp;
    image_array2[lat_index][lon_index] = cs->rh;
    image_array3[lat_index][lon_index] = cs->nep;
    image_array4[lat_index][lon_index] = cs->npp2;

    cs=cs->next_site;
  }
  
  for(i=0;i<((LATMAX)-(LATMIN));i++){
    for(j=0;j<((LONMAX)-(LONMIN));j++){
     fprintf(outfile1,"%f ",image_array1[i][j]);
     fprintf(outfile2,"%f ",image_array2[i][j]);
     fprintf(outfile3,"%f ",image_array3[i][j]);
     fprintf(outfile4,"%f ",image_array4[i][j]);
    }
    fprintf(outfile1,"\n");
    fprintf(outfile2,"\n");
    fprintf(outfile3,"\n");
    fprintf(outfile4,"\n");
  }
  fclose(outfile1);
  fclose(outfile2);
  fclose(outfile3);
  fclose(outfile4);
    
  printf("DONE\n");

  return;
}
/*****************************************************************************/
/**   REGIONAL CANOPY HEIGHT                                              ****/
/*****************************************************************************/
void print_region_max_height(unsigned int t,struct site **siteptr, UserData data){
  int i,j;
  struct site *cs;
  struct patch *cp;
  FILE *outfile;
  char filename[STR_LEN];
  int lat_index,lon_index;
  float maxh;
  
  float image_array[(LATMAX)-(LATMIN)][(LONMAX)-(LONMIN)];

  if(t%PRINT_OUTPUT_FREQ==0){

  printf("PRINT_REGION_MAX_HEIGHT...");
   strcpy(filename,"../output/");
   strcat(filename,data->base_filename);
  strcat(filename,".region.maxh");    
 
  if(t==0)
    outfile=fopen(filename,"w");
  else outfile=fopen(filename,"a");

  for(i=0;i<((LATMAX)-(LATMIN));i++)
    for(j=0;j<((LONMAX)-(LONMIN));j++)
      image_array[i][j] = -9.9;

  cs= *siteptr;
  
  while(cs!=NULL){

    lat_index = (int) LATMAX-(cs->lat+0.5);
    lon_index = (int) (cs->lon-0.5) - (LONMIN);

    maxh = -9.9;
    cp=cs->youngest_patch;
    while(cp != NULL){
      if(cp->tallest != NULL){
	if(cp->tallest->hite > maxh)
	  maxh = cp->tallest->hite;
      }
      cp = cp->older;
    }
	
    image_array[lat_index][lon_index] = maxh;
    cs=cs->next_site;
  }
  
  for(i=0;i<((LATMAX)-(LATMIN));i++){
    for(j=0;j<((LONMAX)-(LONMIN));j++){
     fprintf(outfile,"%f ",image_array[i][j]);
    }
    fprintf(outfile,"\n");
  }
  fclose(outfile);
  
  printf("DONE\n");
  }
  return;
}
/*****************************************************************************/
/**    REGIONAL LAI                                                       ****/
/*****************************************************************************/
void print_region_lai(unsigned int t, struct site **siteptr, UserData data){
  int i,j;
  struct site *cs;
  FILE *outfile;
  char filename[STR_LEN];
  int lat_index,lon_index;
  
  float image_array[(LATMAX)-(LATMIN)][(LONMAX)-(LONMIN)];

  if(t%PRINT_OUTPUT_FREQ==0){
  printf("PRINT_REGION_LAI...");
  strcpy(filename,"../output/");
  strcat(filename,data->base_filename);
  strcat(filename,".region.lai");    
  
  if(t%PRINT_OUTPUT_FREQ == 0)
    outfile=fopen(filename,"w");
  else
    outfile=fopen(filename,"a");
  
  for(i=0;i<((LATMAX)-(LATMIN));i++)
    for(j=0;j<((LONMAX)-(LONMIN));j++)
      image_array[i][j] = -9.9;

  cs= *siteptr;
  
  while(cs!=NULL){

    lat_index = (int) LATMAX-(cs->lat+0.5);
    lon_index = (int) (cs->lon-0.5) - (LONMIN);

    image_array[lat_index][lon_index] = cs->lai;
    cs=cs->next_site;
  }
  
  for(i=0;i<((LATMAX)-(LATMIN));i++){
    for(j=0;j<((LONMAX)-(LONMIN));j++){
     fprintf(outfile,"%f ",image_array[i][j]);
    }
    fprintf(outfile,"\n");
  }
  fclose(outfile);
  
  printf("DONE\n");
  }
  return;
}
/*****************************************************************************/
/* GRID CELL AREA                                                          ***/
/*****************************************************************************/
void print_region_grid_cell_area(struct site **siteptr, UserData data){
  int i,j;
  struct site *cs;
  FILE *outfile;
  char filename[STR_LEN];
  int lat_index,lon_index;
  int lat_index2,lon_index2;
  
  float image_array[(LATMAX)-(LATMIN)][(LONMAX)-(LONMIN)];

  printf("PRINT_REGION_GRID_CELL_AREA...");
  strcpy(filename,"../output/");
  strcat(filename,data->base_filename);
  strcat(filename,".region.grid_cell_area");    
  outfile=fopen(filename,"w");
  
  for(i=0;i<((LATMAX)-(LATMIN));i++)
    for(j=0;j<((LONMAX)-(LONMIN));j++)
      image_array[i][j] = -9.9;

  cs= *siteptr;
  
  while(cs!=NULL){

    lat_index = (int) LATMAX-(cs->lat+0.5);
    lon_index = (int) (cs->lon-0.5) - (LONMIN);

    lat_index2=(int) cs->lat+0.5;
    lat_index2+=90;
    lon_index2=0;

    image_array[lat_index][lon_index] = cs->sdata->area;
    cs=cs->next_site;
  }
  
  for(i=0;i<((LATMAX)-(LATMIN));i++){
    for(j=0;j<((LONMAX)-(LONMIN));j++){
     fprintf(outfile,"%f ",image_array[i][j]);
    }
    fprintf(outfile,"\n");
  }
  fclose(outfile);
  
  printf("DONE\n");

  return;
}
/*****************************************************************************/
/**   REGIONAL ABOVE-GROUND BIOMASS                                       ****/
/*****************************************************************************/
void print_region_agb(unsigned int t,struct site **siteptr, UserData data){
  int i,j;
  struct site *cs;
  FILE *outfile;
  char filename[STR_LEN];
  int lat_index,lon_index;
  
  float image_array[(LATMAX)-(LATMIN)][(LONMAX)-(LONMIN)];

  if(t%PRINT_OUTPUT_FREQ==0){
    printf("PRINT_REGION_AG_BIOMASS...");
    strcpy(filename,"../output/");    
    strcat(filename,data->base_filename);
    strcat(filename,".region.agb");    
    outfile=fopen(filename,"w");
    
    for(i=0;i<((LATMAX)-(LATMIN));i++)
      for(j=0;j<((LONMAX)-(LONMIN));j++)
	image_array[i][j] = -9.9;
    
    cs= *siteptr;
    
    while(cs!=NULL){
      
      lat_index = (int) LATMAX-(cs->lat+0.5);
      lon_index = (int) (cs->lon-0.5) - (LONMIN);
      
    image_array[lat_index][lon_index] = cs->plant_ag_biomass;
    cs=cs->next_site;
    }
    
    for(i=0;i<((LATMAX)-(LATMIN));i++){
      for(j=0;j<((LONMAX)-(LONMIN));j++){
     fprintf(outfile,"%f ",image_array[i][j]);
      }
    fprintf(outfile,"\n");
    }
    fclose(outfile);
    
    printf("DONE\n");
  }
  return;
}
/**************************************************************/
/**    PRINT BIOMASS FOR IMAGE PLOT                        ****/
/**************************************************************/
void print_region_biomass(unsigned int t,struct site **siteptr, UserData data){
  int i,j;
  struct site *cs;
  FILE *outfile;
  char filename[STR_LEN];
  int lat_index,lon_index;
  
  float image_array[(LATMAX)-(LATMIN)][(LONMAX)-(LONMIN)];

  if(t%PRINT_OUTPUT_FREQ==0){
  printf("PRINT_REGION_BIOMASS...");
  strcpy(filename,"../output/");
  strcat(filename,data->base_filename);
  strcat(filename,".region.biomass");    
  
  if(t==0)
    outfile=fopen(filename,"w");
  else outfile=fopen(filename,"a");

  for(i=0;i<((LATMAX)-(LATMIN));i++)
    for(j=0;j<((LONMAX)-(LONMIN));j++)
      image_array[i][j] = -9.9;

  cs= *siteptr;
  
  while(cs!=NULL){

    lat_index = (int) LATMAX-(cs->lat+0.5);
    lon_index = (int) (cs->lon-0.5) - (LONMIN);

    image_array[lat_index][lon_index] = cs->plant_biomass;
    cs=cs->next_site;
  }
  
  for(i=0;i<((LATMAX)-(LATMIN));i++){
    for(j=0;j<((LONMAX)-(LONMIN));j++){
     fprintf(outfile,"%f ",image_array[i][j]);
    }
    fprintf(outfile,"\n");
  }
  fclose(outfile);
  
  printf("DONE\n");

  }
  return;
}
/**************************************************************/
/**    PRINT BIOMASS FOR IMAGE PLOT                        ****/
/**************************************************************/
void print_region_pft_biomass(unsigned int t,struct site **siteptr, UserData data){
  int i,j,k;
  struct site *cs;
  FILE *outfile;
  char filename[STR_LEN],tmpname[STR_LEN];
  int lat_index,lon_index;
  
  float image_array[(LATMAX)-(LATMIN)][(LONMAX)-(LONMIN)];

  if(t%PRINT_OUTPUT_FREQ==0){
    printf("PRINT_REGION_SPP_BIOMASS...");
    
    for(k=0;k<N_PFT;k++){
    strcpy(filename,"../output/");
    strcat(filename,data->base_filename);
    strcat(filename,".region.biomass.");    
    sprintf(tmpname,"pft%d",k);
    strcat(filename,tmpname);
    
    if(t==0)
      outfile=fopen(filename,"w");
    else outfile=fopen(filename,"a");

    for(i=0;i<((LATMAX)-(LATMIN));i++)
      for(j=0;j<((LONMAX)-(LONMIN));j++)
      image_array[i][j] = -9.9;
    
    cs= *siteptr;
    
    while(cs!=NULL){
      
      lat_index = (int) LATMAX-(cs->lat+0.5);
      lon_index = (int) (cs->lon-0.5) - (LONMIN);
      
      image_array[lat_index][lon_index] = cs->pft_biomass[k];
      cs=cs->next_site;
    }
  
    for(i=0;i<((LATMAX)-(LATMIN));i++){
      for(j=0;j<((LONMAX)-(LONMIN));j++){
	fprintf(outfile,"%f ",image_array[i][j]);
      }
      fprintf(outfile,"\n");
    }
    fclose(outfile);
  }
  printf("DONE\n");
  }
  return;
}
/*************************************************************************/
/*** PRINT REGION BASAL AREA             *********************************/
/*************************************************************************/
void print_region_basal_area(unsigned int t,struct site **siteptr, UserData data){
  int i,j;
  struct site *cs;
  FILE *outfile;
  char filename[STR_LEN];
  int lat_index,lon_index;
  
  float image_array[(LATMAX)-(LATMIN)][(LONMAX)-(LONMIN)];
  
  if(t%PRINT_OUTPUT_FREQ==0){
    printf("PRINT_REGION_BASAL_AREA...");
    strcpy(filename,"../output/");    
    strcat(filename,data->base_filename);
    strcat(filename,".region.basal_area");    
    
    if(t==0)
      outfile=fopen(filename,"w");
    else
      outfile=fopen(filename,"a");
    
    for(i=0;i<((LATMAX)-(LATMIN));i++)
      for(j=0;j<((LONMAX)-(LONMIN));j++)
	image_array[i][j] = -9.9;
    
    cs= *siteptr;
    
    while(cs!=NULL){
      
      lat_index = (int) LATMAX-(cs->lat+0.5);
      lon_index = (int) (cs->lon-0.5) - (LONMIN);
      
      image_array[lat_index][lon_index] = cs->basal_area;
      cs=cs->next_site;
    }
    
    for(i=0;i<((LATMAX)-(LATMIN));i++){
      for(j=0;j<((LONMAX)-(LONMIN));j++){
	fprintf(outfile,"%f ",image_array[i][j]);
    }
      fprintf(outfile,"\n");
    }
    fclose(outfile);
    
    printf("DONE\n");
  }
  return;
}
/**************************************************************/
/**    PRINT INTEGRATION MAP FOR IMAGE PLOT                ****/
/**************************************************************/
void print_region_integration_flag(unsigned int t,struct site **siteptr, UserData data){
  int i,j;
  struct site *cs;
  struct patch *cp;
  FILE *outfile;
  char filename[STR_LEN];
  int lat_index,lon_index;
  
  int image_array[(LATMAX)-(LATMIN)][(LONMAX)-(LONMIN)];


  if(t%PRINT_OUTPUT_FREQ==0){
  printf("PRINT_REGION_INTEGRATION_MAP...");
  strcpy(filename,"../output/");
  strcat(filename,data->base_filename);
  strcat(filename,".region.integration_flag");    
  
  if(t==0)
    outfile=fopen(filename,"w");
  else outfile=fopen(filename,"a");
  
  for(i=0;i<((LATMAX)-(LATMIN));i++)
    for(j=0;j<((LONMAX)-(LONMIN));j++)
      image_array[i][j] = -9;

  cs= *siteptr;
  
  while(cs!=NULL){

    lat_index = (int) LATMAX-(cs->lat+0.5);
    lon_index = (int) (cs->lon-0.5) - (LONMIN);

    image_array[lat_index][lon_index] = cs->integration_flag;
    cs=cs->next_site;
  }
  
  for(i=0;i<((LATMAX)-(LATMIN));i++){
    for(j=0;j<((LONMAX)-(LONMIN));j++){
     fprintf(outfile,"%d ",image_array[i][j]);
    }
    fprintf(outfile,"\n");
  }
  fclose(outfile);
  
  printf("DONE\n");
  }
  return;
}

/***************************************************************************/
void print_region_disturbance_rates(unsigned int t, struct site **siteptr,  UserData data){
  /*Function to print matrix for image plots*/

  int i,j,k;
  struct site *cs;
  FILE *outfile1;
  char filename[STR_LEN];
  int lat_index,lon_index;
    
  float image_array1[(LATMAX)-(LATMIN)][(LONMAX)-(LONMIN)];

  printf("PRINT_REGION_DISTURBANCE_RATES...");
  strcpy(filename,"../output/");
  strcat(filename,data->base_filename);
  strcat(filename,".region.disturbance_rate");    
  
  if(t%PRINT_OUTPUT_FREQ == 0){
    outfile1=fopen(filename,"w");
  }
  else{
    outfile1=fopen(filename,"a");
  }

  for(i=0;i<((LATMAX)-(LATMIN));i++){
    for(j=0;j<((LONMAX)-(LONMIN));j++){
      image_array1[i][j] = -9.9;    
    }
  }
  
  cs= *siteptr;
  
  while(cs!=NULL){

    lat_index = (int) LATMAX-(cs->lat+0.5);
    lon_index = (int) (cs->lon-0.5) - (LONMIN);
     
    cs->disturbance_rate = 0.00;
    for(k=0;k<N_DIST_TYPES;k++)
      cs->disturbance_rate += cs->disturbance_rates[k];

    image_array1[lat_index][lon_index] = cs->disturbance_rate;
        
    cs=cs->next_site;
  }
  
  for(i=0;i<((LATMAX)-(LATMIN));i++){
    for(j=0;j<((LONMAX)-(LONMIN));j++){
     fprintf(outfile1,"%f ",image_array1[i][j]);
    }
    fprintf(outfile1,"\n");
  }
  fclose(outfile1);
    
  printf("DONE\n");

  return;
  
}
/***************************************************************************/
/*  PRINT SOIL CHARS IN MATRIX FILES FOR IMAGE PLOTS                     ***/
/***************************************************************************/
void print_region_soil_chars(struct site **siteptr,  UserData data){
  /*Function to print matrix for image plots*/

  int i,j;
  struct site *cs;
  FILE *outfile1, *outfile2, *outfile3, *outfile4, *outfile5;
  char filename[STR_LEN];
  int lat_index,lon_index;
    
  int image_array1[(LATMAX)-(LATMIN)][(LONMAX)-(LONMIN)];
  float image_array2[(LATMAX)-(LATMIN)][(LONMAX)-(LONMIN)];
  float image_array3[(LATMAX)-(LATMIN)][(LONMAX)-(LONMIN)];
  float image_array4[(LATMAX)-(LATMIN)][(LONMAX)-(LONMIN)];

  printf("PRINT_REGION_SOIL_CHARS...");
  strcpy(filename,"../output/");
  strcat(filename,data->base_filename);
  strcat(filename,".region.soil_type");    
  outfile1=fopen(filename,"w");
  strcpy(filename,"../output/");
  strcat(filename,data->base_filename);
  strcat(filename,".region.soil_depth");    
  outfile2=fopen(filename,"w");
  strcpy(filename,"../output/");
  strcat(filename,data->base_filename);
  strcat(filename,".region.soil_theta_max");    
  outfile3=fopen(filename,"w");
  strcpy(filename,"../output/");
  strcat(filename,data->base_filename);
  strcat(filename,".region.soil_ksat");    
  outfile4=fopen(filename,"w");

  for(i=0;i<((LATMAX)-(LATMIN));i++){
    for(j=0;j<((LONMAX)-(LONMIN));j++){
      image_array1[i][j] = 0;
      image_array2[i][j] = -9.9;    
      image_array3[i][j] = -9.9;
      image_array4[i][j] = -9.9;
    }
  }
  
  cs= *siteptr;
  
  while(cs!=NULL){

    lat_index = (int) LATMAX-(cs->lat+0.5);
    lon_index = (int) (cs->lon-0.5) - (LONMIN);
     
    image_array1[lat_index][lon_index] = cs->sdata->soil_type;
    image_array2[lat_index][lon_index] = cs->sdata->soil_depth; 
    image_array3[lat_index][lon_index] = cs->sdata->theta_max; 
    image_array4[lat_index][lon_index] = cs->sdata->k_sat; 
    
    cs=cs->next_site;
  }
  
  for(i=0;i<((LATMAX)-(LATMIN));i++){
    for(j=0;j<((LONMAX)-(LONMIN));j++){
     fprintf(outfile1,"%d ",image_array1[i][j]);
     fprintf(outfile2,"%f ",image_array2[i][j]);
     fprintf(outfile3,"%f ",image_array3[i][j]);
     fprintf(outfile4,"%f ",image_array4[i][j]);
    }
    fprintf(outfile1,"\n");
    fprintf(outfile2,"\n");
    fprintf(outfile3,"\n");
    fprintf(outfile4,"\n");

  }
  fclose(outfile1);
  fclose(outfile2);
  fclose(outfile3);
  fclose(outfile4);
  
  printf("DONE\n");

  return;
  
}
/***************************************************************************/
/***** PRINT REGION SOIL C AND N                                        ****/
/***************************************************************************/
void print_region_soil_C_and_N(unsigned int t,struct site **siteptr,  UserData data){
  /*Function to print matrix files for image plots*/

  int i,j;
  struct site *cs;
  FILE *outfile1, *outfile2, *outfile3;
  char filename1[STR_LEN], filename2[STR_LEN], filename3[STR_LEN];
  int lat_index,lon_index;
    
  float image_array1[(LATMAX)-(LATMIN)][(LONMAX)-(LONMIN)];
  float image_array2[(LATMAX)-(LATMIN)][(LONMAX)-(LONMIN)];
  float image_array3[(LATMAX)-(LATMIN)][(LONMAX)-(LONMIN)];
  

  if(t%PRINT_OUTPUT_FREQ==0){
  printf("PRINT_REGION_SOIL_C_AND_N...");
  strcpy(filename1,"../output/");
  strcat(filename1,data->base_filename);
  strcat(filename1,".region.soil_C");    
  strcpy(filename2,"../output/");
  strcat(filename2,data->base_filename);
  strcat(filename2,".region.soil_N");    
  strcpy(filename3,"../output/");
  strcat(filename3,data->base_filename);
  strcat(filename3,".region.mineralized_soil_N");    
 
  if(t==0){
    outfile1=fopen(filename1,"w");
    outfile2=fopen(filename2,"w");
    outfile3=fopen(filename3,"w");
  }
  else{
    outfile1=fopen(filename1,"a");
    outfile2=fopen(filename2,"a");
    outfile3=fopen(filename3,"a");
  }
 
  for(i=0;i<((LATMAX)-(LATMIN));i++){
    for(j=0;j<((LONMAX)-(LONMIN));j++){
      image_array1[i][j] = -9.9;    
      image_array2[i][j] = -9.9;
      image_array3[i][j] = -9.9;
    }
  }
  
  cs= *siteptr;
  
  while(cs!=NULL){

    lat_index = (int) LATMAX-(cs->lat+0.5);
    lon_index = (int) (cs->lon-0.5) - (LONMIN);
     
    /*total soil C*/
    image_array1[lat_index][lon_index] = cs->fast_soil_C + cs->structural_soil_C + cs->slow_soil_C;
    /*total soil N*/
    image_array2[lat_index][lon_index] = cs->mineralized_soil_N + cs->fast_soil_N + cs->structural_soil_C * 1.0/data->c2n_structural + cs->slow_soil_C * 1.0/data->c2n_slow; 
    /*mineralized soil N*/
    image_array3[lat_index][lon_index] = cs->mineralized_soil_N;
     
    cs=cs->next_site;
  }
  
  for(i=0;i<((LATMAX)-(LATMIN));i++){
    for(j=0;j<((LONMAX)-(LONMIN));j++){
     fprintf(outfile1,"%f ",image_array1[i][j]);
     fprintf(outfile2,"%f ",image_array2[i][j]);
     fprintf(outfile3,"%f ",image_array3[i][j]);
    }
    fprintf(outfile1,"\n");
    fprintf(outfile2,"\n");
    fprintf(outfile3,"\n");
  }
  fclose(outfile1);
  fclose(outfile2);
  fclose(outfile3);
    
  printf("DONE\n");
  }

  return;
  
}
/***************************************************************************/
/*  PRINT PRECIP in MATRIX FILES FOR IMAGE PLOTS                         ***/
/***************************************************************************/
void print_region_precip(struct site **siteptr,  UserData data){
  /*Function to print matrix files of mech.in data for image plots*/

  int m,i,j;
  struct site *cs;
  FILE *outfile1;
  char filename[STR_LEN];
  int lat_index,lon_index;
  
  float image_array1[(LATMAX)-(LATMIN)][(LONMAX)-(LONMIN)];
   
  printf("PRINT_REGION_PRECIPITATION....");
  strcpy(filename,"../output/");
  strcat(filename,data->base_filename);
  strcat(filename,".region.precipitation");    
  outfile1=fopen(filename,"w");
  
  for(i=0;i<((LATMAX)-(LATMIN));i++){
    for(j=0;j<((LONMAX)-(LONMIN));j++){
      image_array1[i][j] = -9.9;
    }
  }
  
  for(m=0;m<N_CLIMATE;m++){

    cs= *siteptr;
    while(cs!=NULL){
      
      lat_index = (int) LATMAX-(cs->lat+0.5);
      lon_index = (int) (cs->lon-0.5) - (LONMIN);
    
      image_array1[lat_index][lon_index] = cs->sdata->precip[m];
    
      cs=cs->next_site;
    }
    
    for(i=0;i<((LATMAX)-(LATMIN));i++){
      for(j=0;j<((LONMAX)-(LONMIN));j++){
	fprintf(outfile1,"%f ",image_array1[i][j]);
      }
      fprintf(outfile1,"\n");
    }
  }

  fclose(outfile1);
    
  printf("DONE\n");
  return;
}
/************************************************************************/
void print_region_temp(struct site **siteptr,  UserData data){
  /*Function to print matrix data for image plots*/

  int m,i,j;
  struct site *cs;
  FILE *outfile1;
  char filename[STR_LEN];
  int lat_index,lon_index;
  
  float image_array1[(LATMAX)-(LATMIN)][(LONMAX)-(LONMIN)];
   
  printf("PRINT_REGION_TEMP....");
  strcpy(filename,"../output/");
  strcat(filename,data->base_filename);
  strcat(filename,".region.temp");    
  outfile1=fopen(filename,"w");
  
  for(i=0;i<((LATMAX)-(LATMIN));i++){
    for(j=0;j<((LONMAX)-(LONMIN));j++){
      image_array1[i][j] = -9.9;
    }
  }
  
  for(m=0;m<N_CLIMATE;m++){

    cs= *siteptr;
    while(cs!=NULL){
      
      lat_index = (int) LATMAX-(cs->lat+0.5);
      lon_index = (int) (cs->lon-0.5) - (LONMIN);
    
      image_array1[lat_index][lon_index] = cs->sdata->temp[m];
    
      cs=cs->next_site;
    }
    
    for(i=0;i<((LATMAX)-(LATMIN));i++){
      for(j=0;j<((LONMAX)-(LONMIN));j++){
	fprintf(outfile1,"%f ",image_array1[i][j]);
      }
      fprintf(outfile1,"\n");
    }
  }

  fclose(outfile1);
    
  printf("DONE\n");
  return;
}
/************************************************************************/
void print_region_soil_temp(struct site **siteptr,  UserData data){
  /*Function to print matrix  data for image plots*/

  int m,i,j;
  struct site *cs;
  FILE *outfile1;
  char filename[STR_LEN];
  int lat_index,lon_index;
  
  float image_array1[(LATMAX)-(LATMIN)][(LONMAX)-(LONMIN)];
   
  printf("PRINT_REGION_SOIL_TEMP....");
  strcpy(filename,"../output/");
  strcat(filename,data->base_filename);
  strcat(filename,".region.soil_temp");    
  outfile1=fopen(filename,"w");
  
  for(i=0;i<((LATMAX)-(LATMIN));i++){
    for(j=0;j<((LONMAX)-(LONMIN));j++){
      image_array1[i][j] = -9.9;
    }
  }
  
  for(m=0;m<N_CLIMATE;m++){

    cs= *siteptr;
    while(cs!=NULL){
      
      lat_index = (int) LATMAX-(cs->lat+0.5);
      lon_index = (int) (cs->lon-0.5) - (LONMIN);
    
      image_array1[lat_index][lon_index] = cs->sdata->soil_temp[m];
    
      cs=cs->next_site;
    }
    
    for(i=0;i<((LATMAX)-(LATMIN));i++){
      for(j=0;j<((LONMAX)-(LONMIN));j++){
	fprintf(outfile1,"%f ",image_array1[i][j]);
      }
      fprintf(outfile1,"\n");
    }
  }

  fclose(outfile1);
    
  printf("DONE\n");
  return;
}
/***************************************************************************/
/*  PRINT An and Anb at top of canopy in MATRIX FILES FOR IMAGE PLOTS    ***/
/***************************************************************************/
void print_region_mech_An(struct site **siteptr,  UserData data){
  /*Function to print matrix files of mech.in data for image plots*/

  int i,j,m;
  struct site *cs;
  FILE *outfile1, *outfile2, *outfile3, *outfile4;
  char filename[STR_LEN];
  int lat_index,lon_index;
  
  float image_array1[(LATMAX)-(LATMIN)][(LONMAX)-(LONMIN)];
  float image_array2[(LATMAX)-(LATMIN)][(LONMAX)-(LONMIN)];
  float image_array3[(LATMAX)-(LATMIN)][(LONMAX)-(LONMIN)];
  float image_array4[(LATMAX)-(LATMIN)][(LONMAX)-(LONMIN)];
  
  printf("PRINT_REGION_MECH_An...");
  strcpy(filename,"../output/");
  strcat(filename,data->base_filename);
  strcat(filename,".region.c3An0");    
  outfile1=fopen(filename,"w");
  strcpy(filename,"../output/");
  strcat(filename,data->base_filename);
  strcat(filename,".region.c4An0");    
  outfile2=fopen(filename,"w");
  strcpy(filename,"../output/");  
  strcat(filename,data->base_filename);
  strcat(filename,".region.c3Anb0");    
  outfile3=fopen(filename,"w");
  strcpy(filename,"../output/");  
  strcat(filename,data->base_filename);
  strcat(filename,".region.c4Anb0");    
  outfile4=fopen(filename,"w");
 
  for(i=0;i<((LATMAX)-(LATMIN));i++){
    for(j=0;j<((LONMAX)-(LONMIN));j++){
      image_array1[i][j] = -9.9;
      image_array2[i][j] = -9.9;
      image_array3[i][j] = -9.9;
      image_array4[i][j] = -9.9;
    }
  }
  
  for(m=0;m<N_CLIMATE;m++){
    cs= *siteptr;
    
    while(cs!=NULL){
      
      lat_index = (int) LATMAX-(cs->lat+0.5);
      lon_index = (int) (cs->lon-0.5) - (LONMIN);
      
      image_array1[lat_index][lon_index] = cs->sdata->Ao[0][m][0];
      image_array2[lat_index][lon_index] = cs->sdata->Ao[0][m][1];
      image_array3[lat_index][lon_index] = cs->sdata->Ac[0][m][0];
      image_array4[lat_index][lon_index] = cs->sdata->Ac[0][m][1];
      
      cs=cs->next_site;
    }
    
    for(i=0;i<((LATMAX)-(LATMIN));i++){
      for(j=0;j<((LONMAX)-(LONMIN));j++){
	fprintf(outfile1,"%f ",image_array1[i][j]);
	fprintf(outfile2,"%f ",image_array2[i][j]);
	fprintf(outfile3,"%f ",image_array3[i][j]);
	fprintf(outfile4,"%f ",image_array4[i][j]);
      }
      fprintf(outfile1,"\n");
      fprintf(outfile2,"\n");
      fprintf(outfile3,"\n");
      fprintf(outfile4,"\n");
    }
  }
  fclose(outfile1);
  fclose(outfile2);
  fclose(outfile3);
  fclose(outfile4);
    
  printf("DONE\n");

  return;
}
/***************************************************************************/
/*  PRINT A LAND SEA MASK MATRIX FILE                                    ***/
/***************************************************************************/
void print_region_mask(struct site **siteptr, UserData data){
  int i,j;
  struct site *cs;
  FILE *outfile;
  char filename[STR_LEN];
  int lat_index,lon_index;
  
  int image_array[(LATMAX)-(LATMIN)][(LONMAX)-(LONMIN)];

  printf("PRINT_REGION_MASK....");
  strcpy(filename,"../output/");
  strcat(filename,data->base_filename);
  strcat(filename,".region.mask");    
  outfile=fopen(filename,"w");
  
  for(i=0;i<((LATMAX)-(LATMIN));i++)
    for(j=0;j<((LONMAX)-(LONMIN));j++)
      image_array[i][j] = 0;

  cs= *siteptr;
  
  while(cs!=NULL){
    
    lat_index = (int) (LATMAX) - (cs->lat+0.5);
    lon_index = (int) (cs->lon-0.5) - (LONMIN);
    
    image_array[lat_index][lon_index] = 1;
    cs=cs->next_site;
  }
 
  for(i=0;i<((LATMAX)-(LATMIN));i++){
    for(j=0;j<((LONMAX)-(LONMIN));j++){
     fprintf(outfile,"%d ",image_array[i][j]);
    }
    fprintf(outfile,"\n");
  }
  fclose(outfile);
  
  printf("DONE\n");

  return;
}

/*****************************************************************************/
#endif /*REGION*/
/*****************************************************************************/






















