/*  read_inputs.c
 *   Copyright (C) 2001 Paul Moorcroft, G.C. Hurtt and S.W. Pacala
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include "ed.h"
/***************************************************************************/
void read_input_data_layers(UserData data){

  read_sois(data);
  printf("first soi = %p \n",data->first_soi);
  read_mask(data);
  printf("first soi = %p \n",data->first_soi);
  read_grid_cell_area(data);
  printf("first soi = %p \n",data->first_soi);
  return;

}
/***************************************************************************/
void read_sois(UserData data){
  
  struct soi *new_soi,*cur_soi;
  int i;
  float lat,lon;
  char name[STR_LEN];
  char filename[STR_LEN];
  FILE *infile;

  printf("read_sois...\n");
  
  strcpy(filename,"../inputs/sois/");
  strcat(filename,"soi.list");

  infile=fopen(filename,"r");

  printf("%s\n",filename);

  if(fscanf(infile,"%*s %d\n",&(data->num_sois)) != 1){
       printf("rmi: Error reading from file: %p\n",infile);
       fclose(infile);
       return;
     } 

  printf("num_sois= %d\n",data->num_sois);

  if(data->num_sois > 0){
    data->first_soi=(struct soi *) malloc (sizeof(struct soi));
    fscanf(infile,"%s %f %f\n",name,&lat,&lon);
    data->first_soi->lat = lat;
    data->first_soi->lon = lon;
    strcpy(data->first_soi->name,name);
    data->first_soi->next_soi = NULL;

    if(data->num_sois > 1){
      cur_soi=data->first_soi;
      for(i=1;i<data->num_sois;i++){
	fscanf(infile,"%s %f %f\n",name,&lat,&lon);
	new_soi=(struct soi *) malloc (sizeof(struct soi));
	new_soi->lat=lat;
	new_soi->lon=lon;
	strcpy(new_soi->name,name);
	new_soi->next_soi=NULL;
	cur_soi->next_soi=new_soi;
	cur_soi=new_soi;
      }
    }      
  }
  fclose(infile);
 
#if DEBUG_READ
  cur_soi=data->first_soi;
  while(cur_soi!=NULL){
    printf("soi = %s lat= %f lon= %f\n",cur_soi->name,cur_soi->lat,cur_soi->lon);
    cur_soi=cur_soi->next_soi;
  }
#endif

  printf("read_sois COMPLETE\n");
  return;
}
/***************************************************************************/
void read_mask(UserData data){
  /* This function reads in a global land/sea mask and stores it in a
     2-D array. This is used to determine is a site should be modeled or not*/
  
  FILE *infile;
  int i,j;

  printf("read_mask...\n");

  infile=fopen("../inputs/mask/land_sea.msk","r");

  for(i=0;i<180;i++){
    for(j=0;j<360;j++){
      fscanf(infile,"%d ",&(data->mask[i][j]));
#if DEBUG_READ
      printf("rm: %d %d %d\n",i,j,data->mask[i][j]);
#endif
    }
  }
  
  fclose(infile);
  return;
  
}

/***************************************************************************/
void read_grid_cell_area(UserData data){

  FILE *infile;
  int i,j;
  char filename[STR_LEN];

  printf("read_grid_cell_area...\n");
  strcpy(filename,"../inputs/grid_cell_area/GRIDAREA.GRD");
  infile=fopen(filename,"r");

  for(i=0;i<180;i++){
    for(j=0;j<360;j++){
      fscanf(infile,"%f ",&(data->grid_cell_area[i][j]));
      data->grid_cell_area[i][j]*=1000000.0; /*convert km2 to m2 */
    }
    fscanf(infile,"\n");
  }
  fclose(infile);
  return;

}
/**************************************************************************/
void read_soil_chars(struct site **first_site){

  /*needs checking*/
  FILE *infile;
  float tlat,tlon;
  float tmp1,tmp2,tmp3,tmp4,tmp5;
  int i,n;
  struct site *siteptr;
  static int filelength; 
  char dummy[10000];
  char filename[STR_LEN];
  
  strcpy(filename,"../inputs/soils/");
  strcat(filename,"amaz.soils.dat");
  filelength=2625;

  siteptr = *first_site;

  if(!(infile=fopen(filename,"r"))){
    printf("rsci: Can't open file: %p \n",infile);
    return;
  }
  /*function to read over header line*/
  fgets(dummy,10000,infile);

  siteptr->sdata->soil_depth=0.00; /*initialize*/
  siteptr->sdata->soil_type=0.0; /*initialize*/
  siteptr->sdata->theta_max=0.0;
  siteptr->sdata->k_sat=0.0;
  siteptr->sdata->tau=0.0;

  n=0;
  for(i=0;i<filelength;i++){	
    if(fscanf(infile,"%f %f %f %f %f %f %f\n",&tlat,&tlon,&tmp1,&tmp2,&tmp3,&tmp4,&tmp5) != 7){
      printf("rsci: Error reading from file: %p\n",infile);
      fclose(infile);
      return;
    }
    
    if((tlat<=siteptr->lat+0.5) && (tlat>siteptr->lat-0.5) && (tlon<=siteptr->lon+0.5) && (tlon>=siteptr->lon-0.5)){
#if DEBUG_READ
       printf("rsci: %s data read from line %d of file \n",siteptr->name,i+2); 
#endif
	n++;
	
	siteptr->sdata->soil_depth=tmp1;
	siteptr->sdata->soil_type=(int) tmp2;
        siteptr->sdata->theta_max = tmp3;
        siteptr->sdata->k_sat = tmp4; 
        siteptr->sdata->tau = tmp5; 
      }
  }

#if DEBUG_READ  
  printf("rsci: %s soil type = %d soil_depth = %f theta_max %f ksat %f \n",siteptr->name,siteptr->sdata->soil_type,siteptr->sdata->soil_depth,siteptr->sdata->theta_max,siteptr->sdata->k_sat); 
#endif

  if(n<1){
    printf("rsci: WARNING! soils data not found for the following site \n");
    printf("rsci: lat %f lon %f\n",siteptr->lat, siteptr->lon);
  }
  
  fclose(infile);
  return;
  
}
/**************************************************************************/
void read_mechanism_lut(struct site **siteptr, UserData data){
  int i,j,index;
  int dummy;
  char filename[STR_LEN];
  char pathname[STR_LEN];
  char tmpname[STR_LEN];
  struct site *cs; 
  FILE *infile;

  cs = *siteptr;

  strcpy(pathname,"../inputs/mechanism.in/");
  
  cs = *siteptr;
  
  strcpy(filename,pathname);
  strcat(filename,"c3/");
  for(index=0;index<PT;index++){
    /*assign filename for mechanism.in file*/
    sprintf(tmpname,"lat%dlon%d.in",(int) (cs->lat+0.5), (int) (cs->lon-0.5)); 
    strcat(filename,tmpname);
    
    if(!(infile=fopen(filename,"r"))){
      printf("rml: Can't open file: %s \n",filename);
      return;
    }
  
    for(j=0;j<N_CLIMATE;j++){
      if(fscanf(infile,"%d%f\n",&dummy,&cs->sdata->tf[j][index])!= 2){
	printf("rml:Error reading tf %d from file: %p\n",j,infile);
	fclose(infile);
	return;
      }
      for(i=0;i<N_LIGHT;i++){
	if(fscanf(infile, "%f",&cs->sdata->light_levels[i][index]) != 1){ 
	  printf("rml:Error reading shade %d %d from file: %p\n",j,i,infile);
	  fclose(infile);
	  return;
	}
#if DEBUG_READ
	 printf("rml:shade =%f \n",cs->sdata->light_levels[i][index]);
#endif      
	if(fscanf(infile,"%f%f%f%f",&cs->sdata->Ao[i][j][index],&cs->sdata->Ac[i][j][index],&cs->sdata->Eo[i][j][index], &cs->sdata->Ec[i][j][index]) != 4){
	 
	  printf("rml:Error reading c3 Amax and c3 Evap %d %d from file: %p\n",j,i,infile); 
	  fclose(infile); 
	  return; 
	}  
#if DEBUG_READ
	printf("rml: pt %d Ao %f Eo %f Ac %f Ec %f \n",index, cs->sdata->Ao[i][j][index],cs->sdata->Eo[i][j][index],cs->sdata->Ac[i][j][index],cs->sdata->Ec[i][j][index]);
#endif
      }   /* end loop over light   */
    }     /* end loop over N_CLIMATE */
    
    fclose(infile);
    printf("rml: pt %d... Done \n",index); 
    
    strcpy(filename,pathname);
    strcat(filename,"c4/");
  }
  return;	
}

/****************************************************************************/
void read_precip(struct site **first_site){
  FILE *infile;
  float tlat,tlon;
  float tmp0,tmp1,tmp2,tmp3,tmp4,tmp5,tmp6,tmp7,tmp8,tmp9,tmp10,tmp11;
  int i,j,n;
  struct site *siteptr;
  static int filelength;
  char filename[STR_LEN];

  siteptr = *first_site;


  strcpy(filename,"../inputs/precip/");
  strcat(filename,"precip.islscp.amaz");
  filelength = 2625;

  if(!(infile=fopen(filename,"r"))){
    printf("rp: Can't open file: %p \n",infile);
    return;
  }

#if DEBUG_READ  
   printf("rp: site lat %f lon %f\n",siteptr->lat,siteptr->lon); 
#endif
  for(j=0;j<N_CLIMATE;j++)
    siteptr->sdata->precip[j]=0.0;
  
  n=0;
  for(i=0;i<filelength;i++){	
    if(fscanf(infile,"%f %f %f %f %f %f %f %f %f %f %f %f %f %f\n",&tlat,&tlon,&tmp0,&tmp1,&tmp2,&tmp3,&tmp4,&tmp5,&tmp6,&tmp7,&tmp8,&tmp9,&tmp10,&tmp11) != 14){
      printf("rp: Error reading from file: %p\n",infile);
      fclose(infile);
      return;
    }    
    
    if((tlat<=siteptr->lat+0.5) && (tlat>siteptr->lat-0.5) && (tlon<=siteptr->lon+0.5) && (tlon>siteptr->lon-0.5))
      { 
	n++;
	siteptr->sdata->precip[0]=tmp0;
	siteptr->sdata->precip[1]=tmp1;
	siteptr->sdata->precip[2]=tmp2;
	siteptr->sdata->precip[3]=tmp3;
	siteptr->sdata->precip[4]=tmp4;
	siteptr->sdata->precip[5]=tmp5;
	siteptr->sdata->precip[6]=tmp6;
	siteptr->sdata->precip[7]=tmp7;
	siteptr->sdata->precip[8]=tmp8;
	siteptr->sdata->precip[9]=tmp9;
	siteptr->sdata->precip[10]=tmp10;
	siteptr->sdata->precip[11]=tmp11;
      }
  }
  siteptr->sdata->precip_average = 0.00;
  for(j=0;j<N_CLIMATE;j++){
    siteptr->sdata->precip[j]*=N_CLIMATE; /*to make units avg mm/yr*/
    siteptr->sdata->precip_average+=siteptr->sdata->precip[j]/(1.0*N_CLIMATE);
  }
  
  if(n<1) printf("rp: Error reading precip for site %s\n",siteptr->name);

  fclose(infile);

  return;
  
}
/****************************************************************************/
void read_soil_temp(struct site **first_site){
  FILE *infile;
  float tlat,tlon;
  float tmp0,tmp1,tmp2,tmp3,tmp4,tmp5,tmp6,tmp7,tmp8,tmp9,tmp10,tmp11;
  int i,j,n;
  struct site *siteptr;
  static int filelength; 
  char filename[STR_LEN];
  
  /*printf("read_soil_temp...\n");*/

  siteptr = *first_site;

  strcpy(filename,"../inputs/soil_temp/");

  strcat(filename,"soil_temp.islscp.amaz");
  filelength=2625;

  if(!(infile=fopen(filename,"r"))){
    printf("rst: Can't open file: %p \n",infile);
    return;
  }
  
  for(j=0;j<N_CLIMATE;j++)
    siteptr->sdata->soil_temp[j]=0.0;
  n=0;
  for(i=0;i<filelength;i++){	
    if(fscanf(infile,"%f %f %f %f %f %f %f %f %f %f %f %f %f %f\n",&tlat,&tlon,&tmp0,&tmp1,&tmp2,&tmp3,&tmp4,&tmp5,&tmp6,&tmp7,&tmp8,&tmp9,&tmp10,&tmp11) != 14){
      printf("rsti: Error reading tf %d from file: %p\n",j,infile);
      fclose(infile);
      return;
    }
#if DEBUG_READ    
     printf("rst: %f %f %f %f %f %f %f %f %f %f %f %f %f %f\n",tlat,tlon,tmp0,tmp1,tmp2,tmp3,tmp4,tmp5,tmp6,tmp7,tmp8,tmp9,tmp10,tmp11);
#endif    
    
    if((tlat<=siteptr->lat+0.5) && (tlat>siteptr->lat-0.5) && (tlon<=siteptr->lon+0.5) && (tlon>siteptr->lon-0.5))
      {
	n++;
        /*subtract 273.2 to convert degrees Kelvin to degrees C*/
	siteptr->sdata->soil_temp[0]= (tmp0 - 273.2); 
	siteptr->sdata->soil_temp[1]= (tmp1 - 273.2);
	siteptr->sdata->soil_temp[2]= (tmp2 - 273.2);
	siteptr->sdata->soil_temp[3]= (tmp3 - 273.2);
	siteptr->sdata->soil_temp[4]= (tmp4 - 273.2);
	siteptr->sdata->soil_temp[5]= (tmp5 - 273.2);
	siteptr->sdata->soil_temp[6]= (tmp6 - 273.2);
	siteptr->sdata->soil_temp[7]= (tmp7 - 273.2);
	siteptr->sdata->soil_temp[8]= (tmp8 - 273.2);
	siteptr->sdata->soil_temp[9]= (tmp9 - 273.2);
	siteptr->sdata->soil_temp[10]= (tmp10 - 273.2);
	siteptr->sdata->soil_temp[11]= (tmp11 - 273.2);
      }
  }
  
  siteptr->sdata->soil_temp_average = 0.00;
  for(j=0;j<N_CLIMATE;j++){
    siteptr->sdata->soil_temp_average+=siteptr->sdata->soil_temp[j]/(1.0*N_CLIMATE);
  }

  if(n<1){
    printf("rsti: WARNING! No input data found\n");
  }
#if DEBUG_READ  
  printf("%s MONTHLY soil-temp. average = %f (C)\n",siteptr->name, siteptr->sdata->soil_temp_average);
#endif   
  fclose(infile);
  return;
    
}
/*****************************************************************************/
void read_temp(struct site **first_site){
  FILE *infile;
  float tlat,tlon;
  float tmp0,tmp1,tmp2,tmp3,tmp4,tmp5,tmp6,tmp7,tmp8,tmp9,tmp10,tmp11;
  int i,j,n;
  int filelength;
  struct site *siteptr;
  char filename[256];

  /* input data was created by averaging islscp 2m_temp 6hrly data */

  printf("read_temp...\n");


 strcpy(filename,"../inputs/temp/");
 strcat(filename,"temp.islscp.amaz");
 filelength=877;

  siteptr = *first_site;
  
  if(!(infile=fopen(filename,"r"))){
    printf("rt: Can't open file: %p \n",infile);
    return;
  }

  for(j=0;j<N_CLIMATE;j++)
    siteptr->sdata->temp[j]=0.0;
  
  siteptr->sdata->temp_average=0.0;
  
  n=0;
  for(i=0;i<filelength;i++){
    fscanf(infile,"%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\n",&tlat,&tlon,&tmp0,&tmp1,&tmp2,&tmp3,&tmp4,&tmp5,&tmp6,&tmp7,&tmp8,&tmp9,&tmp10,&tmp11);

#if DEBUG_READ    
    printf("rt:%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\n",tlat,tlon,tmp0,tmp1,tmp2,tmp3,tmp4,tmp5,tmp6,tmp7,tmp8,tmp9,tmp10,tmp11);
#endif     
    
    if((tlat<=siteptr->lat+0.5) && (tlat>siteptr->lat-0.5) && (tlon<=siteptr->lon+0.5) && (tlon>siteptr->lon-0.5)){
	 n++;
	 siteptr->sdata->temp[0]=tmp0;
	 siteptr->sdata->temp[1]=tmp1;
	 siteptr->sdata->temp[2]=tmp2;
	 siteptr->sdata->temp[3]=tmp3;
	 siteptr->sdata->temp[4]=tmp4;
	 siteptr->sdata->temp[5]=tmp5;
	 siteptr->sdata->temp[6]=tmp6;
	 siteptr->sdata->temp[7]=tmp7;
	 siteptr->sdata->temp[8]=tmp8;
	 siteptr->sdata->temp[9]=tmp9;
	 siteptr->sdata->temp[10]=tmp10;
	 siteptr->sdata->temp[11]=tmp11;
       }
   }

 for(j=0;j<N_CLIMATE;j++){
   siteptr->sdata->temp_average += siteptr->sdata->temp[j]/(1.0*N_CLIMATE);
 }
#if DEBUG_READ 
  printf("rt: %s temp. average = %f \n",siteptr->name, siteptr->sdata->temp_average);
#endif  
 if(n<1){
   printf("rt: WARNING! No input data found for site %s!\n",siteptr->name);
 }
 
#if DEBUG_READ
 printf("LAT %f\t LON %f\t JAN %f\t DEC %f\t N %d\n",siteptr->lat,siteptr->lon,siteptr->sdata->temp[0],siteptr->sdata->temp[11],n);
#endif
 
 fclose(infile);
 return;
}

/****************************************************************************/
/***        read in initial patch distribution from file            ***/
/**********************************************************************/
void read_patch_distribution(struct site **siteptr, UserData data){
  /* read data from patch file (.pss) into a new linked list */

  FILE *infile;
  struct site *cs; 
  struct patch *newp, *cp;
  float old_time;
  float age,area,water,fsc,stsc,stsl,ssc,psc,msn,fsn;
  int dist_type;
  char filename[STR_LEN];
  char old_address[20];
  int count=0;
  char dummy[10000];

  cs=*siteptr; /* assign pointer to site */

  strcpy(filename,"../ic/");
  strcat(filename,data->base_filename);
  strcat(filename,".");
  strcat(filename,cs->name);
  strcat(filename,".pss");

  printf("rpd: Reading in patches for %s from %s \n",cs->name,filename);
  if(!(infile=fopen(filename,"r"))){
    printf("rpd: Can't open file: %p %s \n",infile,filename);
    return;
  }

 /*function to read over header line*/
  fgets(dummy,10000,infile);

 /* read in data elements */
  while(fscanf(infile,"%f%s%d%f%f%f%f%f%f%f%f%f%f\n",&old_time,old_address,&dist_type,&age,&area,&water,&fsc,&stsc,&stsl,&ssc,&psc,&msn,&fsn) != EOF){    
    count++;
    printf("rpd: Initializing patch %d \n",count);
    create_patch(&cs,&newp,dist_type,age,area,water,fsc,stsc,stsl,ssc,psc,msn,fsn,data);    
    
    read_cohort_distribution(&cs,&newp,old_address,data);

    /* first patch in linked list */    
    if(count==1){
      newp->younger=NULL; 
      newp->older=NULL;
      cs->youngest_patch = newp;
      cs->oldest_patch = newp;
      cp = cs->youngest_patch;
    }
    else{
       newp->younger=NULL;
       newp->older =  cp;
       (cp->younger) = newp;
       cp = cp->younger;
       cs->youngest_patch = newp;
    }
    
  } /* end while !EOF */
  
  printf("rpd: number of patches read = %d \n\n",count);
  fclose(infile);
  
  return;

}
/**********************************************************************/
/***        read in initial cohort distribution from file            ***/
/**********************************************************************/
void read_cohort_distribution(struct site **siteptr, struct patch **patchptr, char *paddress1, UserData data){

  FILE *infile;
  char filename[STR_LEN];
  struct site *cs; 
  struct patch *cp; 
  float dbh, hite, nindivs, bdead, balive;
  int  pft;
  char paddress2[20];
  float fdum;
  char cdum[20];
  char dummy[10000];

  int count=0;
  cs=*siteptr; /* assign pointer to site */ 
  cp=*patchptr; /* assign pointer to patch */
  cp->tallest=NULL;
  cp->shortest=NULL;

  /* read data from patch file into newpatch structure */
  strcpy(filename,"../ic/");
  strcat(filename,data->base_filename);
  strcat(filename,".");
  strcat(filename,cs->name);
  strcat(filename,".css");

  printf("rcd: reading cohorts for patch %s \n",paddress1);
  if(!(infile=fopen(filename,"r"))){
    printf("rcd: Can't open file: %p %s \n",infile,filename);
    return;
  }

 /*function to read over header line*/
  fgets(dummy,10000,infile);
 /* read in  data elements */
  while(fscanf(infile,"%f%s%s%f%f%d%f%f%f\n",&fdum,paddress2,cdum,&dbh,&hite,&pft,&nindivs,&bdead,&balive) != EOF){
#if DEBUG_READ
    printf("rcd: %6.3f %p %p %8.4f %8.4f %2d %8.4f %8.4f %8.4f \n",fdum,paddress2,cdum,dbh,hite,pft,nindivs,bdead,balive); 
#endif
    /* compare patch address read from cohort file to old address of patch */
    /* if match create cohort with read-in parameters */

#if DEBUG_READ   
    printf("rcd: padd1 %s padd2 %s\n",paddress1,paddress2);
#endif
    if(strcmp(paddress1,paddress2) == 0){
#if DEBUG_READ
     printf("rcd: match: %s %f %f %d %f %f %f\n",paddress2,dbh,hite,pft,nindivs,bdead,balive); 
#endif
      create_cohort(pft,nindivs,hite,dbh,balive,bdead,&cp,data);
      count++;
   }  /* end if */
 } /* end while */

  printf("rcd: cohorts read= %d \n",count);
  fclose(infile);

  return;

}
/**********************************************************************/









