/*  site.c
 *   Copyright (C) 2001 Paul Moorcroft, G.C. Hurtt and S.W. Pacala
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include "ed.h"
/*************************************************************************/
void update_sites(unsigned int t, struct site **firsts, UserData data){
  
  /*updating all info before graphics and printing*/
  struct site *cs;
  struct patch *cp;
  int n;

  cs=*firsts;
  printf("us: time = %f \n",t*TIMESTEP); 
  n = t/N_CLIMATE;
  if(n % (PRINT_OUTPUT_FREQ/N_CLIMATE)==0){    
  
    /************************************/  
    /* calculate patch level quantities */         
    /************************************/
    cs=*firsts;
    while(cs!=NULL){      
      cp=cs->oldest_patch;
      while(cp!=NULL){
        calculate_patch_biomass(&cp,data);
        light_levels(&cp,data);
        calculate_patch_cfluxes(&cp,data);
	cp=cp->younger;         
      } /* end loop over patches */
      cs=cs->next_site;        	
    } /* end loop over sites */

    /***********************************/
    /* calculate site level quantities */
    /***********************************/
    cs=*firsts;
    while(cs!=NULL){      
      calculate_site_biomass(&cs,data);
      calculate_site_water(&cs,data);
      calculate_site_cfluxes(&cs,data); 
      calculate_site_soil_pools(&cs,data);
      cs=cs->next_site;        	
    } /* end loop over sites */

    /*******************************/
    /* print output files for sois */
    /*******************************/
    cs=*firsts;
    while(cs!=NULL){
#if DEBUG_UPDATE_SITES
      printf("us: %s %d\n",cs->name,cs->soi);
#endif
      if(cs->soi == 1){
	print_site_biomass(t,&cs,data); 
        print_patches(t,&cs,data);
        print_cohorts(t,&cs,data); 
        print_patch_cfluxes(t,&cs,data); 
        print_site_cfluxes(t,&cs,data);   	
        print_site_water(t,&cs,data); 	
	print_diagnostics(t,&cs,data);         
        print_site_soil_pools(t,&cs,data);
	print_patch_light_levels(t,&cs,data);      
	print_patch_water(t,&cs,data);        
      }
      cs=cs->next_site;        	
    } /* end loop over sites */

    /**********************/
    /*print regional files*/
    /**********************/
#if REGION
       print_region_diagnostics(t,firsts,data);
       print_region_agb(t,firsts,data);
       print_region_biomass(t,firsts,data);
       print_region_pft_biomass(t,firsts,data);
       print_region_max_height(t,firsts,data);
       print_region_integration_flag(t,firsts,data);
       print_region_soil_C_and_N(t,firsts,data);
       print_region_basal_area(t,firsts,data);

       /*these record seasonal variation*/
       print_region_disturbance_rates(t,firsts,data);
       print_region_water_losses(t,firsts,data);
       print_region_lai(t,firsts,data);
       print_region_carbon_fluxes(t,firsts,data);
       print_region_water(t,firsts,data);  
       /* print_region_nep3(t,firsts,data); */

#endif /*REGION*/
  } 
    /**********************/
    /*print restart files*/
    /**********************/
  if((t>0) && (t%PRINT_RESTART_FREQ == 0)){
    cs= *firsts;
    while(cs != NULL){
      print_system_state(t,&cs,data);
      cs = cs->next_site;
    }
  }

  return;
}
/***************************************************************************/
void calculate_site_water(struct site **siteptr, UserData data){

  struct site *cs;
  struct patch *cp;    
  cs=*siteptr;

    /*init*/
    cs->water=0.0;
    cs->theta=0.0;
    cs->plant_water_uptake=0.0;
    cs->plant_water_demand=0.0;
    cs->perc=0.0;
    cs->soil_evap=0.0;        

    cp = cs->youngest_patch;
    while(cp != NULL){
	cs->water += cp->water*cp->area/AREA;
	cs->theta += cp->theta*cp->area/AREA;
	cs->plant_water_uptake += cp->plant_water_uptake*cp->area/AREA;
	cs->plant_water_demand += cp->plant_water_demand*cp->area/AREA;      
	cs->perc += cp->perc*cp->area/AREA;  
	cs->soil_evap += cp->soil_evap*cp->area/AREA;      
        cp=cp->older;     
    }  /* end loop over patches */
    
  return;
}
/**************************************************************/
void calculate_site_biomass(struct site **siteptr,UserData data){  
  struct site *cs;
  struct patch *cp;
  int i;
  
  cs=*siteptr;
  
  /*init*/
  for(i=0;i<N_PFT;i++){
    cs->pft_biomass[i] = 0.00;
    cs->pft_ag_biomass[i] = 0.00;
    cs->pft_basal_area[i] = 0.00;
  }
  cs->plant_ag_biomass = 0.00;
  cs->plant_biomass = 0.00;
  cs->basal_area = 0.00;
  cs->lai = 0.00;

  cp= cs->youngest_patch;
  while(cp != NULL){
    for(i=0;i<N_PFT;i++){
      cs->pft_biomass[i] += cp->pft_biomass[i]*cp->area/AREA;
      cs->pft_ag_biomass[i] += cp->pft_ag_biomass[i]*cp->area/AREA;
      cs->pft_basal_area[i] += cp->pft_basal_area[i]*cp->area/AREA;
    } /* end loop over pfts */

    cs->plant_biomass += cp->plant_biomass*cp->area/AREA;
    cs->plant_ag_biomass += cp->plant_ag_biomass*cp->area/AREA;
    cs->basal_area += cp->basal_area*cp->area/AREA;
    cs->lai += cp->lai*cp->area/AREA;

    cp=cp->older;
  }

  /* print to files */
#if DEBUG_UPDATE_SITES
  fprintf(stdout,"%s soil_c %8.5f tb %8.5f tagb %8.5f ba %8.5f lai %8.5f ",cs->name,cs->soil_c,cs->plant_biomass,cs->plant_ag_biomass,cs->basal_area,cs->lai);
  for(i=0;i<N_PFT;i++){
    fprintf(stdout,"pft %d b %8.6f agb %8.6f ba %8.6f ",i ,cs->pft_biomass[i],cs->pft_ag_biomass[i],cs->pft_basal_area[i]); 
  }
  fprintf(stdout,"\n");
#endif


  return;
  
}
/*****************************************************************************/

/*****************************************************/
void calculate_site_cfluxes(struct site **siteptr, UserData data){

  struct site *cs;
  struct patch *cp;

  cs= *siteptr; 
  /*init*/    
  cs->gpp = 0.0;
  cs->npp=0.0;
  cs->nep=0.0;
  cs->rh=0.0;
  cs->npp2=0.0;
  cs->aa_npp=0.0; 
  cs->aa_npp2=0.0; 
  cs->aa_nep=0.0; 
  cs->aa_rh=0.0;
 
  cp=cs->youngest_patch;
  while(cp != NULL){
    cs->gpp += cp->gpp * cp->area/AREA;
    cs->npp += cp->npp * cp->area/AREA;
    cs->npp2 += cp->npp2 * cp->area/AREA;
    cs->nep += cp->nep * cp->area/AREA;
    cs->rh += cp->rh * cp->area/AREA;
    cs->aa_gpp += cp->aa_gpp*cp->area/AREA;
    cs->aa_npp += cp->aa_npp*cp->area/AREA;
    cs->aa_npp2 += cp->aa_npp2 * cp->area/AREA;
    cs->aa_nep += cp->aa_nep*cp->area/AREA; 
    cs->aa_rh += cp->aa_rh*cp->area/AREA; 
    cp=cp->older;
  } /* end loop over patches */

  return;
}
/*************************************************/
/*****************************************************************************/
void calculate_site_soil_pools(struct site **siteptr, UserData data){

  struct site *cs;
  struct patch *cp;
    
  cs=*siteptr;
    
  /*calculate total_sc*/
  /*init*/
  cs->soil_c=0.00;
  cs->litter=0.00;
  cs->fast_soil_C=0.00;
  cs->structural_soil_C=0.00;
  cs->slow_soil_C=0.00;
  cs->mineralized_soil_N=0.00;
  cs->fast_soil_N=0.00;
  cs->structural_soil_L=0.00;

  cp=cs->youngest_patch;
  while(cp != NULL) { /* loop over sites */
    cs->soil_c += cp->soil_c*cp->area/AREA;
    cs->litter += cp->litter*cp->area/AREA;
    cs->fast_soil_C += cp->fast_soil_C*cp->area/AREA;
    cs->structural_soil_C += cp->structural_soil_C*cp->area/AREA;
    cs->slow_soil_C += cp->slow_soil_C*cp->area/AREA;;
    cs->mineralized_soil_N += cp->mineralized_soil_N*cp->area/AREA;
    cs->fast_soil_N += cp->fast_soil_N*cp->area/AREA;
    cs->structural_soil_L += cp->structural_soil_L*cp->area/AREA;
   
    cp=cp->older;
  }

  return;
}
/****************************************************/
/*************************************************/
void calculate_patch_cfluxes(struct patch **patchptr,UserData data){

  struct patch *cp;
  struct cohort *cc;
  
  cp = *patchptr;

      /*init*/
      if(data->time_period == 0){ /* reset annual averages */
        cp->aa_npp2=0.0; 
        cp->aa_npp=0.0; 
        cp->aa_nep=0.0; 
        cp->aa_rh=0.0;
        cp->aa_gpp=0.0;
      }

      cp->npp = 0.0;
      cp->nep = 0.0;
      cp->npp2 = 0.0;
      cp->gpp = 0.0;
      cc = cp->shortest;
      while(cc != NULL){
        if(cc->hite >= data->hgt_min[cc->pft]){
	cp->npp += cc->npp*(cc->n/cp->area);
        cp->nep += cc->npp*(cc->n/cp->area);
	cp->npp2 += cc->npp2*(cc->n/cp->area);
        cp->gpp += cc->gpp*(cc->n/cp->area);
	}
        cc = cc->taller;
      } /* end loop over cohorts */
      cp->nep -= cp->rh;

      /* acculmulate annual averages */
      cp->aa_npp += cp->npp*data->deltat;
      cp->aa_npp2 += cp->npp2*data->deltat;
      cp->aa_nep += (cp->npp - cp->rh)*data->deltat;
      cp->aa_rh  += cp->rh*data->deltat;
      cp->aa_gpp += cp->gpp*data->deltat;
      
  return;
}
/************************************************/
void calculate_patch_biomass(struct patch **patchptr,UserData data){

  struct patch *cp;
  struct cohort *cc;
  int i;
  
  cp = *patchptr;  
    /* init patch biomass bins */
    for(i=0;i<N_PFT;i++){
      cp->pft_biomass[i]=0.00;
      cp->pft_ag_biomass[i]=0.00;
      cp->pft_basal_area[i]=0.00;
    }
    cp->plant_ag_biomass=0.00;
    cp->plant_biomass=0.00;
    cp->basal_area=0.00;            
    cp->lai=0.00; 
    cc = cp->shortest;
    while(cc != NULL){
      if((cc->dbh >= MIN_DBH_CLASS)&&(cc->hite > MIN_HGT_CLASS)){
	cp->pft_biomass[cc->pft]+=cc->b*(cc->n/cp->area);
	cp->pft_ag_biomass[cc->pft]+=cc->babove*(cc->n/cp->area);
 	cp->plant_ag_biomass+= cc->babove*(cc->n/cp->area);
	cp->plant_biomass += cc->b*(cc->n/cp->area);
	cp->basal_area+= (M_PI/4.0)*pow(cc->dbh,2.0)*(cc->n/cp->area);
	cp->pft_basal_area[cc->pft]+= (M_PI/4.0)*pow(cc->dbh,2.0)*(cc->n/cp->area);      
	cp->lai += cc->lai;
      } /* end if */
      cc = cc->taller;
    } /* end loop over cohorts */

#if DEBUG_UPDATE_SITES
  /* print to files */
  fprintf(stdout,"patch %p tb %8.5f tagb %8.5f ba %8.5f lai %8.5f ",cp,cp->plant_biomass,cp->plant_ag_biomass,cp->basal_area,cp->lai);
  for(i=0;i<N_PFT;i++){
    fprintf(stdout,"pft %d b %8.6f agb %8.6f ba %8.6f ",i ,cp->pft_biomass[i],cp->pft_ag_biomass[i],cp->pft_basal_area[i]); 
  }
  fprintf(stdout,"\n");  
#endif
  return;
}
/***************************************************************************/
/*** binned site size profiles                                            **/
/***************************************************************************/
void pft_site_size_profile(struct site **pcs, int nbins, UserData data){
   /* sum patch profiles to get pft distributions at site level */
   struct site  *cs;
   struct patch *cp;
   unsigned int i,j;

   /* printf("sssp: \n"); */
   
   cs=*pcs;
   for(i=0;i<N_PFT;i++){  
     for(j=0;j<N_DBH_BINS;j++){ 
       cs->pft_density_profile[i][j]=0.0;
       cs->pft_basal_area_profile[i][j]=0.0;
       cs->pft_agb_profile[i][j]=0.0;
     }
   }

   cp=cs->oldest_patch;
   while(cp != NULL){
     for(i=0;i<N_PFT;i++){  
       for(j=0;j<N_DBH_BINS;j++){ 
       
	 cs->pft_density_profile[i][j]+= cp->pft_density_profile[i][j]*cp->area/AREA;
	 cs->pft_basal_area_profile[i][j]+= cp->pft_basal_area_profile[i][j]*cp->area/AREA;
	 cs->pft_agb_profile[i][j]+= cp->pft_agb_profile[i][j]*cp->area/AREA; 
       }
     }
     cp = cp->younger; 
   } /* end loop over patches */

   /* size profile summed over all pft */
   for(j=0;j<N_DBH_BINS;j++){ 
     cs->density_profile[j]=0.0;
     cs->basal_area_profile[j]=0.0;
     cs->agb_profile[j]=0.0;
   }
   for(j=0;j<N_DBH_BINS;j++){ 
     for(i=0;i<N_PFT;i++){  
       cs->density_profile[j]+= cs->pft_density_profile[i][j];
       cs->basal_area_profile[j]+= cs->pft_basal_area_profile[i][j];
       cs->agb_profile[j]+= cs->pft_agb_profile[i][j];
   }}
   
   /* printf("exit sssp: \n"); */
   return;
}
/*****************************************************************************/










